@@ -5,13 +5,13 @@
 struct SQLexer
 {
 	~SQLexer();
-	SQLexer(SQSharedState *ss,SQLEXREADFUNC rg,SQUserPointer up,CompilerErrorFunc efunc,void *ed);
-	NORETURN void Error(const SQChar *err);
+	SQLexer(SQSharedState *ss,SQLEXREADFUNC rg,SQUserPointer up);
+	[[noreturn]] void Error(const SQChar *err);
 	SQInteger Lex();
 	const SQChar *Tok2Str(SQInteger tok);
 private:
 	SQInteger GetIDType(SQChar *s);
-	SQInteger ReadString(WChar ndelim,bool verbatim);
+	SQInteger ReadString(char32_t ndelim,bool verbatim);
 	SQInteger ReadNumber();
 	void LexBlockComment();
 	SQInteger ReadID();
@@ -19,7 +19,7 @@ private:
 	SQInteger _curtoken;
 	SQTable *_keywords;
 	void INIT_TEMP_STRING() { _longstr.resize(0); }
-	void APPEND_CHAR(WChar c);
+	void APPEND_CHAR(char32_t c);
 	void TERMINATE_BUFFER() { _longstr.push_back('\0'); }
 
 public:
@@ -32,11 +32,9 @@ public:
 	SQFloat _fvalue;
 	SQLEXREADFUNC _readf;
 	SQUserPointer _up;
-	WChar _currdata;
+	char32_t _currdata;
 	SQSharedState *_sharedstate;
 	sqvector<SQChar> _longstr;
-	CompilerErrorFunc _errfunc;
-	void *_errtarget;
 };
 
 #endif
