@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -60,33 +58,34 @@ enum AirportMovingDataFlags {
 
 /** Movement States on Airports (headings target) */
 enum AirportMovementStates {
-	TO_ALL         =  0, ///< Go in this direction for every target.
-	HANGAR         =  1, ///< Heading for hangar.
-	TERM1          =  2, ///< Heading for terminal 1.
-	TERM2          =  3, ///< Heading for terminal 2.
-	TERM3          =  4, ///< Heading for terminal 3.
-	TERM4          =  5, ///< Heading for terminal 4.
-	TERM5          =  6, ///< Heading for terminal 5.
-	TERM6          =  7, ///< Heading for terminal 6.
-	HELIPAD1       =  8, ///< Heading for helipad 1.
-	HELIPAD2       =  9, ///< Heading for helipad 2.
-	TAKEOFF        = 10, ///< Airplane wants to leave the airport.
-	STARTTAKEOFF   = 11, ///< Airplane has arrived at a runway for take-off.
-	ENDTAKEOFF     = 12, ///< Airplane has reached end-point of the take-off runway.
-	HELITAKEOFF    = 13, ///< Helicopter wants to leave the airport.
-	FLYING         = 14, ///< %Vehicle is flying in the air.
-	LANDING        = 15, ///< Airplane wants to land.
-	ENDLANDING     = 16, ///< Airplane wants to finish landing.
-	HELILANDING    = 17, ///< Helicopter wants to land.
-	HELIENDLANDING = 18, ///< Helicopter wants to finish landing.
-	TERM7          = 19, ///< Heading for terminal 7.
-	TERM8          = 20, ///< Heading for terminal 8.
-	HELIPAD3       = 21, ///< Heading for helipad 3.
-	MAX_HEADINGS   = 21, ///< Last valid target to head for.
+	TO_ALL         =   0, ///< Go in this direction for every target.
+	HANGAR         =   1, ///< Heading for hangar.
+	TERM1          =   2, ///< Heading for terminal 1.
+	TERM2          =   3, ///< Heading for terminal 2.
+	TERM3          =   4, ///< Heading for terminal 3.
+	TERM4          =   5, ///< Heading for terminal 4.
+	TERM5          =   6, ///< Heading for terminal 5.
+	TERM6          =   7, ///< Heading for terminal 6.
+	HELIPAD1       =   8, ///< Heading for helipad 1.
+	HELIPAD2       =   9, ///< Heading for helipad 2.
+	TAKEOFF        =  10, ///< Airplane wants to leave the airport.
+	STARTTAKEOFF   =  11, ///< Airplane has arrived at a runway for take-off.
+	ENDTAKEOFF     =  12, ///< Airplane has reached end-point of the take-off runway.
+	HELITAKEOFF    =  13, ///< Helicopter wants to leave the airport.
+	FLYING         =  14, ///< %Vehicle is flying in the air.
+	LANDING        =  15, ///< Airplane wants to land.
+	ENDLANDING     =  16, ///< Airplane wants to finish landing.
+	HELILANDING    =  17, ///< Helicopter wants to land.
+	HELIENDLANDING =  18, ///< Helicopter wants to finish landing.
+	TERM7          =  19, ///< Heading for terminal 7.
+	TERM8          =  20, ///< Heading for terminal 8.
+	HELIPAD3       =  21, ///< Heading for helipad 3.
+	MAX_HEADINGS   =  21, ///< Last valid target to head for.
+	TERMGROUP      = 255, ///< Aircraft is looking for a free terminal in a terminalgroup.
 };
 
 /** Movement Blocks on Airports blocks (eg_airport_flags). */
-static const uint64
+static const uint64_t
 	TERM1_block              = 1ULL <<  0, ///< Block belonging to terminal 1.
 	TERM2_block              = 1ULL <<  1, ///< Block belonging to terminal 2.
 	TERM3_block              = 1ULL <<  2, ///< Block belonging to terminal 3.
@@ -130,10 +129,10 @@ static const uint64
 
 /** A single location on an airport where aircraft can move to. */
 struct AirportMovingData {
-	int16 x;                 ///< x-coordinate of the destination.
-	int16 y;                 ///< y-coordinate of the destination.
-	uint16 flag;             ///< special flags when moving towards the destination.
-	DirectionByte direction; ///< Direction to turn the aircraft after reaching the destination.
+	int16_t x;             ///< x-coordinate of the destination.
+	int16_t y;             ///< y-coordinate of the destination.
+	uint16_t flag;         ///< special flags when moving towards the destination.
+	Direction direction; ///< Direction to turn the aircraft after reaching the destination.
 };
 
 AirportMovingData RotateAirportMovingData(const AirportMovingData *orig, Direction rotation, uint num_tiles_x, uint num_tiles_y);
@@ -190,7 +189,7 @@ DECLARE_ENUM_AS_BIT_SET(AirportFTAClass::Flags)
 /** Internal structure used in openttd - Finite sTate mAchine --> FTA */
 struct AirportFTA {
 	AirportFTA *next;        ///< possible extra movement choices from this position
-	uint64 block;            ///< 64 bit blocks (st->airport.flags), should be enough for the most complex airports
+	uint64_t block;            ///< 64 bit blocks (st->airport.flags), should be enough for the most complex airports
 	byte position;           ///< the position that an airplane is at
 	byte next_position;      ///< next position from this position
 	byte heading;            ///< heading (current orders), guiding an airplane to its target on an airport
