@@ -14,16 +14,28 @@
 #include "command_type.h"
 #include "viewport_type.h"
 #include "station_map.h"
+#include "timer/timer_game_calendar.h"
 
 typedef Pool<BaseStation, StationID, 32, 64000> StationPool;
 extern StationPool _station_pool;
 
 struct StationSpecList {
 	const StationSpec *spec;
-	uint32 grfid;      ///< GRF ID of this custom station
-	uint8  localidx;   ///< Station ID within GRF of station
+	uint32_t grfid;      ///< GRF ID of this custom station
+	uint16_t localidx; ///< Station ID within GRF of station
 };
 
+struct RoadStopSpecList {
+	const RoadStopSpec *spec;
+	uint32_t grfid;      ///< GRF ID of this custom road stop
+	uint16_t localidx; ///< Station ID within GRF of road stop
+};
+
+struct RoadStopTileData {
+	TileIndex tile;
+	uint8_t random_bits;
+	uint8_t animation_frame;
+};
 
 /** StationRect - used to track station spread out rectangle - cheaper than scanning whole map */
 struct StationRect : public Rect {
@@ -62,18 +74,23 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 	Owner owner;                    ///< The owner of this station
 	StationFacility facilities;     ///< The facilities that this station has
 
-	std::vector<StationSpecList> speclist; ///< List of rail station specs of this station.
+	std::vector<StationSpecList> speclist;           ///< List of rail station specs of this station.
+	std::vector<RoadStopSpecList> roadstop_speclist; ///< List of road stop specs of this station
 
-	Date build_date;                ///< Date of construction
+	TimerGameCalendar::Date build_date; ///< Date of construction
 
-	uint16 random_bits;             ///< Random bits assigned to this station
+	uint16_t random_bits;             ///< Random bits assigned to this station
 	byte waiting_triggers;          ///< Waiting triggers (NewGRF) for this station
-	uint8 cached_anim_triggers;     ///< NOSAVE: Combined animation trigger bitmask, used to determine if trigger processing should happen.
-	CargoTypes cached_cargo_triggers; ///< NOSAVE: Combined cargo trigger bitmask
+	uint8_t cached_anim_triggers;                ///< NOSAVE: Combined animation trigger bitmask, used to determine if trigger processing should happen.
+	uint8_t cached_roadstop_anim_triggers;       ///< NOSAVE: Combined animation trigger bitmask for road stops, used to determine if trigger processing should happen.
+	CargoTypes cached_cargo_triggers;          ///< NOSAVE: Combined cargo trigger bitmask
+	CargoTypes cached_roadstop_cargo_triggers; ///< NOSAVE: Combined cargo trigger bitmask for road stops
 
 	TileArea train_station;         ///< Tile area the train 'station' part covers
 	StationRect rect;               ///< NOSAVE: Station spread out rectangle maintained by StationRect::xxx() functions
 
+	std::vector<RoadStopTileData> custom_roadstop_tile_data; ///< List of custom road stop tile data
+
 	/**
 	 * Initialize the base station.
 	 * @param tile The location of the station sign
@@ -101,18 +118,18 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 	 * @param available will return false if ever the variable asked for does not exist
 	 * @return the value stored in the corresponding variable
 	 */
-	virtual uint32 GetNewGRFVariable(const struct ResolverObject &object, byte variable, byte parameter, bool *available) const = 0;
+	virtual uint32_t GetNewGRFVariable(const struct ResolverObject &object, byte variable, byte parameter, bool *available) const = 0;
 
 	/**
 	 * Update the coordinated of the sign (as shown in the viewport).
 	 */
 	virtual void UpdateVirtCoord() = 0;
 
-	inline const char *GetCachedName() const
+	inline const std::string &GetCachedName() const
 	{
-		if (!this->name.empty()) return this->name.c_str();
+		if (!this->name.empty()) return this->name;
 		if (this->cached_name.empty()) this->FillCachedName();
-		return this->cached_name.c_str();
+		return this->cached_name;
 	}
 
 	virtual void MoveSign(TileIndex new_xy)
@@ -167,6 +184,30 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 		return (this->facilities & ~FACIL_WAYPOINT) != 0;
 	}
 
+	inline byte GetRoadStopRandomBits(TileIndex tile) const
+	{
+		for (const RoadStopTileData &tile_data : this->custom_roadstop_tile_data) {
+			if (tile_data.tile == tile) return tile_data.random_bits;
+		}
+		return 0;
+	}
+
+	inline byte GetRoadStopAnimationFrame(TileIndex tile) const
+	{
+		for (const RoadStopTileData &tile_data : this->custom_roadstop_tile_data) {
+			if (tile_data.tile == tile) return tile_data.animation_frame;
+		}
+		return 0;
+	}
+
+private:
+	void SetRoadStopTileData(TileIndex tile, byte data, bool animation);
+
+public:
+	inline void SetRoadStopRandomBits(TileIndex tile, byte random_bits) { this->SetRoadStopTileData(tile, random_bits, false); }
+	inline void SetRoadStopAnimationFrame(TileIndex tile, byte frame) { this->SetRoadStopTileData(tile, frame, true); }
+	void RemoveRoadStopTileData(TileIndex tile);
+
 	static void PostDestructor(size_t index);
 
 private:
