@@ -29,11 +29,11 @@
 /** Base methods for 32bpp SSE blitters. */
 class Blitter_32bppSSE_Base {
 public:
-	virtual ~Blitter_32bppSSE_Base() {}
+	virtual ~Blitter_32bppSSE_Base() = default;
 
 	struct MapValue {
-		uint8 m;
-		uint8 v;
+		uint8_t m;
+		uint8_t v;
 	};
 	static_assert(sizeof(MapValue) == 2);
 
@@ -65,18 +65,18 @@ public:
 
 	/** Data stored about a (single) sprite. */
 	struct SpriteInfo {
-		uint32 sprite_offset;    ///< The offset to the sprite data.
-		uint32 mv_offset;        ///< The offset to the map value data.
-		uint16 sprite_line_size; ///< The size of a single line (pitch).
-		uint16 sprite_width;     ///< The width of the sprite.
+		uint32_t sprite_offset;    ///< The offset to the sprite data.
+		uint32_t mv_offset;        ///< The offset to the map value data.
+		uint16_t sprite_line_size; ///< The size of a single line (pitch).
+		uint16_t sprite_width;     ///< The width of the sprite.
 	};
 	struct SpriteData {
 		SpriteFlags flags;
-		SpriteInfo infos[ZOOM_LVL_COUNT];
+		SpriteInfo infos[ZOOM_LVL_END];
 		byte data[]; ///< Data, all zoomlevels.
 	};
 
-	Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
+	Sprite *Encode(const SpriteLoader::SpriteCollection &sprite, AllocatorProc *allocator);
 };
 
 DECLARE_ENUM_AS_BIT_SET(Blitter_32bppSSE_Base::SpriteFlags);
@@ -88,7 +88,7 @@ public:
 	template <BlitterMode mode, Blitter_32bppSSE_Base::ReadMode read_mode, Blitter_32bppSSE_Base::BlockType bt_last, bool translucent>
 	void Draw(const Blitter::BlitterParams *bp, ZoomLevel zoom);
 
-	Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator) override {
+	Sprite *Encode(const SpriteLoader::SpriteCollection &sprite, AllocatorProc *allocator) override {
 		return Blitter_32bppSSE_Base::Encode(sprite, allocator);
 	}
 
