@@ -1,5 +1,3 @@
-/* $Id: 8bpp_optimized.hpp 26209 2014-01-02 22:41:58Z rubidium $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -16,25 +14,25 @@
 #include "factory.hpp"
 
 /** 8bpp blitter optimised for speed. */
-class Blitter_8bppOptimized FINAL : public Blitter_8bppBase {
+class Blitter_8bppOptimized final : public Blitter_8bppBase {
 public:
 	/** Data stored about a (single) sprite. */
 	struct SpriteData {
-		uint32 offset[ZOOM_LVL_COUNT]; ///< Offsets (from .data) to streams for different zoom levels.
+		uint32_t offset[ZOOM_LVL_END]; ///< Offsets (from .data) to streams for different zoom levels.
 		byte data[];                   ///< Data, all zoomlevels.
 	};
 
-	/* virtual */ void Draw(Blitter::BlitterParams *bp, BlitterMode mode, ZoomLevel zoom);
-	/* virtual */ Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
+	void Draw(Blitter::BlitterParams *bp, BlitterMode mode, ZoomLevel zoom) override;
+	Sprite *Encode(const SpriteLoader::SpriteCollection &sprite, AllocatorProc *allocator) override;
 
-	/* virtual */ const char *GetName() { return "8bpp-optimized"; }
+	const char *GetName() override { return "8bpp-optimized"; }
 };
 
 /** Factory for the 8bpp blitter optimised for speed. */
 class FBlitter_8bppOptimized : public BlitterFactory {
 public:
 	FBlitter_8bppOptimized() : BlitterFactory("8bpp-optimized", "8bpp Optimized Blitter (compression + all-ZoomLevel cache)") {}
-	/* virtual */ Blitter *CreateInstance() { return new Blitter_8bppOptimized(); }
+	Blitter *CreateInstance() override { return new Blitter_8bppOptimized(); }
 };
 
 #endif /* BLITTER_8BPP_OPTIMIZED_HPP */
