@@ -35,25 +35,25 @@ enum Owner : byte {
 	COMPANY_SPECTATOR       = 255, ///< The client is spectating
 };
 DECLARE_POSTFIX_INCREMENT(Owner)
+DECLARE_ENUM_AS_ADDABLE(Owner)
 
 static const uint MAX_LENGTH_PRESIDENT_NAME_CHARS = 32; ///< The maximum length of a president name in characters including '\0'
 static const uint MAX_LENGTH_COMPANY_NAME_CHARS   = 32; ///< The maximum length of a company name in characters including '\0'
 
 static const uint MAX_HISTORY_QUARTERS            = 24; ///< The maximum number of quarters kept as performance's history
-static const uint MAX_COMPANY_SHARE_OWNERS        =  4; ///< The maximum number of shares of a company that can be owned by another company.
 
-/** Define basic enum properties */
-template <> struct EnumPropsT<Owner> : MakeEnumPropsT<Owner, byte, OWNER_BEGIN, OWNER_END, INVALID_OWNER> {};
+static const uint MIN_COMPETITORS_INTERVAL = 0;   ///< The minimum interval (in minutes) between competitors.
+static const uint MAX_COMPETITORS_INTERVAL = 500; ///< The maximum interval (in minutes) between competitors.
 
 typedef Owner CompanyID;
 
-typedef uint16 CompanyMask;
+typedef uint16_t CompanyMask;
 
 struct Company;
-typedef uint32 CompanyManagerFace; ///< Company manager face bits, info see in company_manager_face.h
+typedef uint32_t CompanyManagerFace; ///< Company manager face bits, info see in company_manager_face.h
 
 /** The reason why the company was removed. */
-enum CompanyRemoveReason : uint8 {
+enum CompanyRemoveReason : uint8_t {
 	CRR_MANUAL,    ///< The company is manually removed.
 	CRR_AUTOCLEAN, ///< The company is removed due to autoclean.
 	CRR_BANKRUPT,  ///< The company went belly-up.
@@ -64,7 +64,7 @@ enum CompanyRemoveReason : uint8 {
 };
 
 /** The action to do with CMD_COMPANY_CTRL. */
-enum CompanyCtrlAction : uint8 {
+enum CompanyCtrlAction : uint8_t {
 	CCA_NEW,    ///< Create a new company.
 	CCA_NEW_AI, ///< Create a new AI company.
 	CCA_DELETE, ///< Delete a company.
