@@ -10,12 +10,6 @@
 #ifndef CPU_H
 #define CPU_H
 
-/**
- * Get the tick counter from the CPU (high precision timing).
- * @return The count.
- */
-uint64 ottd_rdtsc();
-
 /**
  * Get the CPUID information from the CPU.
  * @param info The retrieved info. All zeros on architectures without CPUID.
