@@ -10,7 +10,7 @@
 #ifndef DATE_GUI_H
 #define DATE_GUI_H
 
-#include "date_type.h"
+#include "timer/timer_game_calendar.h"
 #include "window_type.h"
 
 /**
@@ -18,8 +18,8 @@
  * @param w the window that sends the callback
  * @param date the date that has been chosen
  */
-typedef void SetDateCallback(const Window *w, Date date, void *data);
+typedef void SetDateCallback(const Window *w, TimerGameEconomy::Date date, void *data);
 
-void ShowSetDateWindow(Window *parent, int window_number, Date initial_date, Year min_year, Year max_year, SetDateCallback *callback, void *callback_data);
+void ShowSetDateWindow(Window *parent, int window_number, TimerGameEconomy::Date initial_date, TimerGameEconomy::Year min_year, TimerGameEconomy::Year max_year, SetDateCallback *callback, void *callback_data);
 
 #endif /* DATE_GUI_H */
