@@ -38,7 +38,7 @@ static bool IncrementSprite(EffectVehicle *v, SpriteID last)
 
 static void ChimneySmokeInit(EffectVehicle *v)
 {
-	uint32 r = Random();
+	uint32_t r = Random();
 	v->sprite_cache.sprite_seq.Set(SPR_CHIMNEY_SMOKE_0 + GB(r, 0, 3));
 	v->progress = GB(r, 16, 3);
 }
@@ -274,8 +274,8 @@ static const BulldozerMovement _bulldozer_movement[] = {
 };
 
 static const struct {
-	int8 x;
-	int8 y;
+	int8_t x;
+	int8_t y;
 } _inc_by_dir[] = {
 	{ -1,  0 },
 	{  0,  1 },
@@ -317,9 +317,9 @@ static void BubbleInit(EffectVehicle *v)
 }
 
 struct BubbleMovement {
-	int8 x:4;
-	int8 y:4;
-	int8 z:4;
+	int8_t x:4;
+	int8_t y:4;
+	int8_t z:4;
 	byte image:4;
 };
 
@@ -621,8 +621,8 @@ EffectVehicle *CreateEffectVehicle(int x, int y, int z, EffectVehicleType type)
  */
 EffectVehicle *CreateEffectVehicleAbove(int x, int y, int z, EffectVehicleType type)
 {
-	int safe_x = Clamp(x, 0, MapMaxX() * TILE_SIZE);
-	int safe_y = Clamp(y, 0, MapMaxY() * TILE_SIZE);
+	int safe_x = Clamp(x, 0, Map::MaxX() * TILE_SIZE);
+	int safe_y = Clamp(y, 0, Map::MaxY() * TILE_SIZE);
 	return CreateEffectVehicle(x, y, GetSlopePixelZ(safe_x, safe_y) + z, type);
 }
 
