@@ -12,8 +12,10 @@
 
 #include "strings_type.h"
 #include "company_type.h"
+#include "command_type.h"
 #include "core/geometry_type.hpp"
-#include "guitimer_func.h"
+
+#include <chrono>
 
 struct GRFFile;
 
@@ -28,21 +30,20 @@ enum WarningLevel {
 /** The data of the error message. */
 class ErrorMessageData {
 protected:
-	GUITimer display_timer;         ///< Timer before closing the message.
-	uint64 decode_params[20];       ///< Parameters of the message strings.
-	const char *strings[20];        ///< Copies of raw strings that were used.
+	bool is_critical;               ///< Whether the error message is critical.
+	std::vector<StringParameterBackup> params; ///< Backup of parameters of the message strings.
 	const GRFFile *textref_stack_grffile; ///< NewGRF that filled the #TextRefStack for the error message.
-	uint textref_stack_size;        ///< Number of uint32 values to put on the #TextRefStack for the error message.
-	uint32 textref_stack[16];       ///< Values to put on the #TextRefStack for the error message.
+	uint textref_stack_size;        ///< Number of uint32_t values to put on the #TextRefStack for the error message.
+	uint32_t textref_stack[16];       ///< Values to put on the #TextRefStack for the error message.
 	StringID summary_msg;           ///< General error message showed in first line. Must be valid.
 	StringID detailed_msg;          ///< Detailed error message showed in second line. Can be #INVALID_STRING_ID.
+	StringID extra_msg;             ///< Extra error message shown in third line. Can be #INVALID_STRING_ID.
 	Point position;                 ///< Position of the error message window.
 	CompanyID face;                 ///< Company belonging to the face being shown. #INVALID_COMPANY if no face present.
 
 public:
 	ErrorMessageData(const ErrorMessageData &data);
-	~ErrorMessageData();
-	ErrorMessageData(StringID summary_msg, StringID detailed_msg, uint duration = 0, int x = 0, int y = 0, const GRFFile *textref_stack_grffile = nullptr, uint textref_stack_size = 0, const uint32 *textref_stack = nullptr);
+	ErrorMessageData(StringID summary_msg, StringID detailed_msg, bool is_critical = false, int x = 0, int y = 0, const GRFFile *textref_stack_grffile = nullptr, uint textref_stack_size = 0, const uint32_t *textref_stack = nullptr, StringID extra_msg = INVALID_STRING_ID);
 
 	/* Remove the copy assignment, as the default implementation will not do the right thing. */
 	ErrorMessageData &operator=(ErrorMessageData &rhs) = delete;
@@ -50,16 +51,21 @@ public:
 	/** Check whether error window shall display a company manager face */
 	bool HasFace() const { return face != INVALID_COMPANY; }
 
-	void SetDParam(uint n, uint64 v);
+	void SetDParam(uint n, uint64_t v);
 	void SetDParamStr(uint n, const char *str);
 	void SetDParamStr(uint n, const std::string &str);
 
 	void CopyOutDParams();
 };
 
+/** Define a queue with errors. */
+typedef std::list<ErrorMessageData> ErrorList;
+
+void ScheduleErrorMessage(ErrorList &datas);
 void ScheduleErrorMessage(const ErrorMessageData &data);
 
-void ShowErrorMessage(StringID summary_msg, StringID detailed_msg, WarningLevel wl, int x = 0, int y = 0, const GRFFile *textref_stack_grffile = nullptr, uint textref_stack_size = 0, const uint32 *textref_stack = nullptr);
+void ShowErrorMessage(StringID summary_msg, int x, int y, CommandCost cc);
+void ShowErrorMessage(StringID summary_msg, StringID detailed_msg, WarningLevel wl, int x = 0, int y = 0, const GRFFile *textref_stack_grffile = nullptr, uint textref_stack_size = 0, const uint32_t *textref_stack = nullptr, StringID extra_msg = INVALID_STRING_ID);
 bool HideActiveErrorMessage();
 
 void ClearErrorMessages();
