@@ -89,7 +89,7 @@ enum FiosType {
  */
 inline AbstractFileType GetAbstractFileType(FiosType fios_type)
 {
-	return static_cast<AbstractFileType>(fios_type & FT_MASK);
+	return static_cast<AbstractFileType>(static_cast<uint>(fios_type) & FT_MASK);
 }
 
 /**
@@ -121,6 +121,7 @@ enum Subdirectory {
 	GAME_DIR,      ///< Subdirectory for all game scripts
 	GAME_LIBRARY_DIR, ///< Subdirectory for all GS libraries
 	SCREENSHOT_DIR,   ///< Subdirectory for all screenshots
+	SOCIAL_INTEGRATION_DIR, ///< Subdirectory for all social integration plugins
 	NUM_SUBDIRS,   ///< Number of subdirectories
 	NO_DIRECTORY,  ///< A path without any base directory
 };
