@@ -15,6 +15,7 @@
 #include "../core/math_func.hpp"
 #include "../zoom_func.h"
 #include "../fileio_func.h"
+#include "../error_func.h"
 #include "truetypefontcache.h"
 
 #include "../table/control_codes.h"
@@ -33,16 +34,17 @@ private:
 	FT_Face face;  ///< The font face associated with this font.
 
 	void SetFontSize(FontSize fs, FT_Face face, int pixels);
-	const void *InternalGetFontTable(uint32 tag, size_t &length) override;
+	const void *InternalGetFontTable(uint32_t tag, size_t &length) override;
 	const Sprite *InternalGetGlyph(GlyphID key, bool aa) override;
 
 public:
 	FreeTypeFontCache(FontSize fs, FT_Face face, int pixels);
 	~FreeTypeFontCache();
 	void ClearFontCache() override;
-	GlyphID MapCharToGlyph(WChar key) override;
+	GlyphID MapCharToGlyph(char32_t key, bool allow_fallback = true) override;
 	std::string GetFontName() override { return fmt::format("{}, {}", face->family_name, face->style_name); }
 	bool IsBuiltInFont() override { return false; }
+	const void *GetOSHandle() override { return &face; }
 };
 
 FT_Library _library = nullptr;
@@ -61,7 +63,7 @@ FreeTypeFontCache::FreeTypeFontCache(FontSize fs, FT_Face face, int pixels) : Tr
 	this->SetFontSize(fs, face, pixels);
 }
 
-void FreeTypeFontCache::SetFontSize(FontSize fs, FT_Face face, int pixels)
+void FreeTypeFontCache::SetFontSize(FontSize, FT_Face, int pixels)
 {
 	if (pixels == 0) {
 		/* Try to determine a good height based on the minimal height recommended by the font. */
@@ -113,6 +115,42 @@ void FreeTypeFontCache::SetFontSize(FontSize fs, FT_Face face, int pixels)
 	}
 }
 
+static FT_Error LoadFont(FontSize fs, FT_Face face, const char *font_name, uint size)
+{
+	Debug(fontcache, 2, "Requested '{}', using '{} {}'", font_name, face->family_name, face->style_name);
+
+	/* Attempt to select the unicode character map */
+	FT_Error error = FT_Select_Charmap(face, ft_encoding_unicode);
+	if (error == FT_Err_Ok) goto found_face; // Success
+
+	if (error == FT_Err_Invalid_CharMap_Handle) {
+		/* Try to pick a different character map instead. We default to
+		 * the first map, but platform_id 0 encoding_id 0 should also
+		 * be unicode (strange system...) */
+		FT_CharMap found = face->charmaps[0];
+		int i;
+
+		for (i = 0; i < face->num_charmaps; i++) {
+			FT_CharMap charmap = face->charmaps[i];
+			if (charmap->platform_id == 0 && charmap->encoding_id == 0) {
+				found = charmap;
+			}
+		}
+
+		if (found != nullptr) {
+			error = FT_Set_Charmap(face, found);
+			if (error == FT_Err_Ok) goto found_face;
+		}
+	}
+
+	FT_Done_Face(face);
+	return error;
+
+found_face:
+	new FreeTypeFontCache(fs, face, size);
+	return FT_Err_Ok;
+}
+
 /**
  * Loads the freetype font.
  * First type to load the fontname as if it were a path. If that fails,
@@ -128,7 +166,7 @@ void LoadFreeTypeFont(FontSize fs)
 
 	if (_library == nullptr) {
 		if (FT_Init_FreeType(&_library) != FT_Err_Ok) {
-			ShowInfoF("Unable to initialize FreeType, using sprite fonts instead");
+			ShowInfo("Unable to initialize FreeType, using sprite fonts instead");
 			return;
 		}
 
@@ -139,21 +177,15 @@ void LoadFreeTypeFont(FontSize fs)
 	FT_Face face = nullptr;
 
 	/* If font is an absolute path to a ttf, try loading that first. */
-	FT_Error error = FT_New_Face(_library, font_name, 0, &face);
-
-#if defined(WITH_COCOA)
-	extern void MacOSRegisterExternalFont(const char *file_path);
-	if (error == FT_Err_Ok) MacOSRegisterExternalFont(font_name);
-#endif
+	int32_t index = 0;
+	if (settings->os_handle != nullptr) index = *static_cast<const int32_t *>(settings->os_handle);
+	FT_Error error = FT_New_Face(_library, font_name, index, &face);
 
 	if (error != FT_Err_Ok) {
 		/* Check if font is a relative filename in one of our search-paths. */
 		std::string full_font = FioFindFullPath(BASE_DIR, font_name);
 		if (!full_font.empty()) {
 			error = FT_New_Face(_library, full_font.c_str(), 0, &face);
-#if defined(WITH_COCOA)
-			if (error == FT_Err_Ok) MacOSRegisterExternalFont(full_font.c_str());
-#endif
 		}
 	}
 
@@ -161,40 +193,40 @@ void LoadFreeTypeFont(FontSize fs)
 	if (error != FT_Err_Ok) error = GetFontByFaceName(font_name, &face);
 
 	if (error == FT_Err_Ok) {
-		Debug(fontcache, 2, "Requested '{}', using '{} {}'", font_name, face->family_name, face->style_name);
-
-		/* Attempt to select the unicode character map */
-		error = FT_Select_Charmap(face, ft_encoding_unicode);
-		if (error == FT_Err_Ok) goto found_face; // Success
-
-		if (error == FT_Err_Invalid_CharMap_Handle) {
-			/* Try to pick a different character map instead. We default to
-			 * the first map, but platform_id 0 encoding_id 0 should also
-			 * be unicode (strange system...) */
-			FT_CharMap found = face->charmaps[0];
-			int i;
-
-			for (i = 0; i < face->num_charmaps; i++) {
-				FT_CharMap charmap = face->charmaps[i];
-				if (charmap->platform_id == 0 && charmap->encoding_id == 0) {
-					found = charmap;
-				}
-			}
-
-			if (found != nullptr) {
-				error = FT_Set_Charmap(face, found);
-				if (error == FT_Err_Ok) goto found_face;
-			}
+		error = LoadFont(fs, face, font_name, settings->size);
+		if (error != FT_Err_Ok) {
+			ShowInfo("Unable to use '{}' for {} font, FreeType reported error 0x{:X}, using sprite font instead", font_name, FontSizeToName(fs), error);
 		}
+	} else {
+		FT_Done_Face(face);
+	}
+}
+
+/**
+ * Load a TrueType font from a file.
+ * @param fs The font size to load.
+ * @param file_name Path to the font file.
+ * @param size Requested font size.
+ */
+void LoadFreeTypeFont(FontSize fs, const std::string &file_name, uint size)
+{
+	if (_library == nullptr) {
+		if (FT_Init_FreeType(&_library) != FT_Err_Ok) {
+			ShowInfo("Unable to initialize FreeType, using sprite fonts instead");
+			return;
+		}
+
+		Debug(fontcache, 2, "Initialized");
 	}
 
-	FT_Done_Face(face);
-
-	ShowInfoF("Unable to use '%s' for %s font, FreeType reported error 0x%X, using sprite font instead", font_name, FontSizeToName(fs), error);
-	return;
-
-found_face:
-	new FreeTypeFontCache(fs, face, settings->size);
+	FT_Face face = nullptr;
+	int32_t index = 0;
+	FT_Error error = FT_New_Face(_library, file_name.c_str(), index, &face);
+	if (error == FT_Err_Ok) {
+		LoadFont(fs, face, file_name.c_str(), size);
+	} else {
+		FT_Done_Face(face);
+	}
 }
 
 
@@ -236,12 +268,13 @@ const Sprite *FreeTypeFontCache::InternalGetGlyph(GlyphID key, bool aa)
 	uint height = std::max(1U, (uint)slot->bitmap.rows  + shadow);
 
 	/* Limit glyph size to prevent overflows later on. */
-	if (width > MAX_GLYPH_DIM || height > MAX_GLYPH_DIM) usererror("Font glyph is too large");
+	if (width > MAX_GLYPH_DIM || height > MAX_GLYPH_DIM) UserError("Font glyph is too large");
 
 	/* FreeType has rendered the glyph, now we allocate a sprite and copy the image into it */
-	SpriteLoader::Sprite sprite;
-	sprite.AllocateData(ZOOM_LVL_NORMAL, width * height);
-	sprite.type = ST_FONT;
+	SpriteLoader::SpriteCollection spritecollection;
+	SpriteLoader::Sprite &sprite = spritecollection[ZOOM_LVL_NORMAL];
+	sprite.AllocateData(ZOOM_LVL_NORMAL, static_cast<size_t>(width) * height);
+	sprite.type = SpriteType::Font;
 	sprite.colours = (aa ? SCC_PAL | SCC_ALPHA : SCC_PAL);
 	sprite.width = width;
 	sprite.height = height;
@@ -270,7 +303,7 @@ const Sprite *FreeTypeFontCache::InternalGetGlyph(GlyphID key, bool aa)
 	}
 
 	GlyphEntry new_glyph;
-	new_glyph.sprite = BlitterFactory::GetCurrentBlitter()->Encode(&sprite, SimpleSpriteAlloc);
+	new_glyph.sprite = BlitterFactory::GetCurrentBlitter()->Encode(spritecollection, SimpleSpriteAlloc);
 	new_glyph.width  = slot->advance.x >> 6;
 
 	this->SetGlyphPtr(key, &new_glyph);
@@ -279,18 +312,20 @@ const Sprite *FreeTypeFontCache::InternalGetGlyph(GlyphID key, bool aa)
 }
 
 
-GlyphID FreeTypeFontCache::MapCharToGlyph(WChar key)
+GlyphID FreeTypeFontCache::MapCharToGlyph(char32_t key, bool allow_fallback)
 {
 	assert(IsPrintable(key));
 
-	if (key >= SCC_SPRITE_START && key <= SCC_SPRITE_END) {
+	FT_UInt glyph = FT_Get_Char_Index(this->face, key);
+
+	if (glyph == 0 && allow_fallback && key >= SCC_SPRITE_START && key <= SCC_SPRITE_END) {
 		return this->parent->MapCharToGlyph(key);
 	}
 
-	return FT_Get_Char_Index(this->face, key);
+	return glyph;
 }
 
-const void *FreeTypeFontCache::InternalGetFontTable(uint32 tag, size_t &length)
+const void *FreeTypeFontCache::InternalGetFontTable(uint32_t tag, size_t &length)
 {
 	FT_ULong len = 0;
 	FT_Byte *result = nullptr;
@@ -315,10 +350,10 @@ void UninitFreeType()
 	_library = nullptr;
 }
 
-#if !defined(_WIN32) && !defined(__APPLE__) && !defined(WITH_FONTCONFIG) && !defined(WITH_COCOA)
+#if !defined(WITH_FONTCONFIG)
 
 FT_Error GetFontByFaceName(const char *font_name, FT_Face *face) { return FT_Err_Cannot_Open_Resource; }
 
-#endif /* !defined(_WIN32) && !defined(__APPLE__) && !defined(WITH_FONTCONFIG) && !defined(WITH_COCOA) */
+#endif /* !defined(WITH_FONTCONFIG) */
 
 #endif /* WITH_FREETYPE */
