@@ -1,9 +1,9 @@
 /*
-* This file is part of OpenTTD.
-* OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
-* OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-* See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
-*/
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file framerate_type.h
  * Types for recording game performance data.
@@ -80,7 +80,7 @@ enum PerformanceElement {
 DECLARE_POSTFIX_INCREMENT(PerformanceElement)
 
 /** Type used to hold a performance timing measurement */
-typedef uint64 TimingMeasurement;
+typedef uint64_t TimingMeasurement;
 
 /**
  * RAII class for measuring simple elements of performance.
@@ -121,5 +121,6 @@ public:
 };
 
 void ShowFramerateWindow();
+void ProcessPendingPerformanceMeasurements();
 
 #endif /* FRAMERATE_TYPE_H */
