@@ -21,7 +21,7 @@ enum LandscapeGenerator {
 	LG_TERRAGENESIS = 1,  ///< TerraGenesis Perlin landscape generator
 };
 
-static const uint32 GENERATE_NEW_SEED = UINT32_MAX; ///< Create a new random seed
+static const uint32_t GENERATE_NEW_SEED = UINT32_MAX; ///< Create a new random seed
 
 /** Modes for GenerateWorld */
 enum GenWorldMode {
@@ -96,7 +96,7 @@ void SetGeneratingWorldProgress(GenWorldProgress cls, uint total);
 void IncreaseGeneratingWorldProgress(GenWorldProgress cls);
 void PrepareGenerateWorldProgress();
 void ShowGenerateWorldProgress();
-void StartNewGameWithoutGUI(uint32 seed);
+void StartNewGameWithoutGUI(uint32_t seed);
 void ShowCreateScenario();
 void StartScenarioEditor();
 
