@@ -14,9 +14,9 @@ public:
 	DemandCalculator(LinkGraphJob &job);
 
 private:
-	int32 max_distance; ///< Maximum distance possible on the map.
-	int32 mod_dist;     ///< Distance modifier, determines how much demands decrease with distance.
-	int32 accuracy;     ///< Accuracy of the calculation.
+	int32_t max_distance; ///< Maximum distance possible on the map.
+	int32_t mod_dist;     ///< Distance modifier, determines how much demands decrease with distance.
+	int32_t accuracy;     ///< Accuracy of the calculation.
 
 	template<class Tscaler>
 	void CalcDemand(LinkGraphJob &job, Tscaler scaler);
@@ -32,12 +32,12 @@ public:
 	 * Call the demand calculator on the given component.
 	 * @param job Component to calculate the demands for.
 	 */
-	virtual void Run(LinkGraphJob &job) const { DemandCalculator c(job); }
+	void Run(LinkGraphJob &job) const override { DemandCalculator c(job); }
 
 	/**
 	 * Virtual destructor has to be defined because of virtual Run().
 	 */
-	virtual ~DemandHandler() {}
+	virtual ~DemandHandler() = default;
 };
 
 #endif /* DEMANDS_H */
