@@ -11,10 +11,7 @@
 #define LRUCACHE_HPP
 
 #include <utility>
-#include <list>
-#include <functional>
 #include <unordered_map>
-#include <stdexcept>
 
 /**
  * Size limited cache with a least recently used eviction strategy.
