@@ -23,22 +23,22 @@ extern NetworkClientSocketPool _networkclientsocket_pool;
 /** Class for handling the server side of the game connection. */
 class ServerNetworkGameSocketHandler : public NetworkClientSocketPool::PoolItem<&_networkclientsocket_pool>, public NetworkGameSocketHandler, public TCPListenHandler<ServerNetworkGameSocketHandler, PACKET_SERVER_FULL, PACKET_SERVER_BANNED> {
 protected:
-	NetworkRecvStatus Receive_CLIENT_JOIN(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_GAME_INFO(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_GAME_PASSWORD(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_COMPANY_PASSWORD(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_GETMAP(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_MAP_OK(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_ACK(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_COMMAND(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_CHAT(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_SET_PASSWORD(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_SET_NAME(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_QUIT(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_ERROR(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_RCON(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_NEWGRFS_CHECKED(Packet *p) override;
-	NetworkRecvStatus Receive_CLIENT_MOVE(Packet *p) override;
+	NetworkRecvStatus Receive_CLIENT_JOIN(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_GAME_INFO(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_GAME_PASSWORD(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_COMPANY_PASSWORD(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_GETMAP(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_MAP_OK(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_ACK(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_COMMAND(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_CHAT(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_SET_PASSWORD(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_SET_NAME(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_QUIT(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_ERROR(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_RCON(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_NEWGRFS_CHECKED(Packet &p) override;
+	NetworkRecvStatus Receive_CLIENT_MOVE(Packet &p) override;
 
 	NetworkRecvStatus SendGameInfo();
 	NetworkRecvStatus SendNewGRFCheck();
@@ -64,18 +64,18 @@ public:
 
 	byte lag_test;               ///< Byte used for lag-testing the client
 	byte last_token;             ///< The last random token we did send to verify the client is listening
-	uint32 last_token_frame;     ///< The last frame we received the right token
+	uint32_t last_token_frame;     ///< The last frame we received the right token
 	ClientStatus status;         ///< Status of this client
-	CommandQueue outgoing_queue; ///< The command-queue awaiting delivery
+	CommandQueue outgoing_queue; ///< The command-queue awaiting delivery; conceptually more a bucket to gather commands in, after which the whole bucket is sent to the client.
 	size_t receive_limit;        ///< Amount of bytes that we can receive at this moment
 
-	struct PacketWriter *savegame; ///< Writer used to write the savegame.
+	std::shared_ptr<struct PacketWriter> savegame; ///< Writer used to write the savegame.
 	NetworkAddress client_address; ///< IP-address of the client (so they can be banned)
 
 	ServerNetworkGameSocketHandler(SOCKET s);
 	~ServerNetworkGameSocketHandler();
 
-	virtual Packet *ReceivePacket() override;
+	std::unique_ptr<Packet> ReceivePacket() override;
 	NetworkRecvStatus CloseConnection(NetworkRecvStatus status) override;
 	std::string GetClientName() const;
 
@@ -87,17 +87,17 @@ public:
 	NetworkRecvStatus SendQuit(ClientID client_id);
 	NetworkRecvStatus SendShutdown();
 	NetworkRecvStatus SendNewGame();
-	NetworkRecvStatus SendRConResult(uint16 colour, const std::string &command);
+	NetworkRecvStatus SendRConResult(uint16_t colour, const std::string &command);
 	NetworkRecvStatus SendMove(ClientID client_id, CompanyID company_id);
 
 	NetworkRecvStatus SendClientInfo(NetworkClientInfo *ci);
 	NetworkRecvStatus SendError(NetworkErrorCode error, const std::string &reason = {});
-	NetworkRecvStatus SendChat(NetworkAction action, ClientID client_id, bool self_send, const std::string &msg, int64 data);
+	NetworkRecvStatus SendChat(NetworkAction action, ClientID client_id, bool self_send, const std::string &msg, int64_t data);
 	NetworkRecvStatus SendExternalChat(const std::string &source, TextColour colour, const std::string &user, const std::string &msg);
 	NetworkRecvStatus SendJoin(ClientID client_id);
 	NetworkRecvStatus SendFrame();
 	NetworkRecvStatus SendSync();
-	NetworkRecvStatus SendCommand(const CommandPacket *cp);
+	NetworkRecvStatus SendCommand(const CommandPacket &cp);
 	NetworkRecvStatus SendCompanyUpdate();
 	NetworkRecvStatus SendConfigUpdate();
 
@@ -120,6 +120,7 @@ public:
 };
 
 void NetworkServer_Tick(bool send_frame);
+void ChangeNetworkRestartTime(bool reset);
 void NetworkServerSetCompanyPassword(CompanyID company_id, const std::string &password, bool already_hashed = true);
 void NetworkServerUpdateCompanyPassworded(CompanyID company_id, bool passworded);
 
