@@ -10,6 +10,8 @@
 #ifndef NETWORK_TYPE_H
 #define NETWORK_TYPE_H
 
+#include "../core/enum_type.hpp"
+
 /** How many clients can we have */
 static const uint MAX_CLIENTS = 255;
 
@@ -37,24 +39,24 @@ enum NetworkVehicleType {
  * Game type the server can be using.
  * Used on the network protocol to communicate with Game Coordinator.
  */
-enum ServerGameType : uint8 {
+enum ServerGameType : uint8_t {
 	SERVER_GAME_TYPE_LOCAL = 0,
 	SERVER_GAME_TYPE_PUBLIC,
 	SERVER_GAME_TYPE_INVITE_ONLY,
 };
 
 /** 'Unique' identifier to be given to clients */
-enum ClientID : uint32 {
+enum ClientID : uint32_t {
 	INVALID_CLIENT_ID = 0, ///< Client is not part of anything
 	CLIENT_ID_SERVER  = 1, ///< Servers always have this ID
 	CLIENT_ID_FIRST   = 2, ///< The first client ID
 };
 
 /** Indices into the client tables */
-typedef uint8 ClientIndex;
+typedef uint8_t ClientIndex;
 
 /** Indices into the admin tables. */
-typedef uint8 AdminIndex;
+typedef uint8_t AdminIndex;
 
 /** Maximum number of allowed admins. */
 static const AdminIndex MAX_ADMINS = 16;
@@ -63,15 +65,15 @@ static const AdminIndex INVALID_ADMIN_ID = UINT8_MAX;
 
 /** Simple calculated statistics of a company */
 struct NetworkCompanyStats {
-	uint16 num_vehicle[NETWORK_VEH_END];            ///< How many vehicles are there of this type?
-	uint16 num_station[NETWORK_VEH_END];            ///< How many stations are there of this type?
+	uint16_t num_vehicle[NETWORK_VEH_END];            ///< How many vehicles are there of this type?
+	uint16_t num_station[NETWORK_VEH_END];            ///< How many stations are there of this type?
 	bool ai;                                        ///< Is this company an AI
 };
 
 /** Some state information of a company, especially for servers */
 struct NetworkCompanyState {
 	std::string password; ///< The password for the company
-	uint16 months_empty;  ///< How many months the company is empty
+	uint16_t months_empty;  ///< How many months the company is empty
 };
 
 struct NetworkClientInfo;
@@ -86,11 +88,12 @@ enum NetworkPasswordType {
  * Destination of our chat messages.
  * @warning The values of the enum items are part of the admin network API. Only append at the end.
  */
-enum DestType {
+enum DestType : byte {
 	DESTTYPE_BROADCAST, ///< Send message/notice to all clients (All)
 	DESTTYPE_TEAM,      ///< Send message/notice to everyone playing the same company (Team)
 	DESTTYPE_CLIENT,    ///< Send message/notice to only a certain client (Private)
 };
+DECLARE_ENUM_AS_ADDABLE(DestType)
 
 /**
  * Actions that can be used for NetworkTextMessage.
