@@ -14,11 +14,8 @@
 
 void NetworkUDPInitialize();
 void NetworkUDPSearchGame();
-void NetworkUDPQueryMasterServer();
-void NetworkUDPQueryServer(NetworkAddress address, bool manually = false);
-void NetworkUDPAdvertise();
-void NetworkUDPRemoveAdvertise(bool blocking);
 void NetworkUDPClose();
+void NetworkUDPServerListen();
 void NetworkBackgroundUDPLoop();
 
 #endif /* NETWORK_UDP_H */
