@@ -10,16 +10,16 @@
 #ifndef NEWGRF_ANIMATION_TYPE_H
 #define NEWGRF_ANIMATION_TYPE_H
 
-static const uint8 ANIM_STATUS_NON_LOOPING  = 0x00; ///< Animation is not looping.
-static const uint8 ANIM_STATUS_LOOPING      = 0x01; ///< Animation is looping.
-static const uint8 ANIM_STATUS_NO_ANIMATION = 0xFF; ///< There is no animation.
+static const uint8_t ANIM_STATUS_NON_LOOPING  = 0x00; ///< Animation is not looping.
+static const uint8_t ANIM_STATUS_LOOPING      = 0x01; ///< Animation is looping.
+static const uint8_t ANIM_STATUS_NO_ANIMATION = 0xFF; ///< There is no animation.
 
 /** Information about animation. */
 struct AnimationInfo {
-	uint8  frames;   ///< The number of frames.
-	uint8  status;   ///< Status; 0: no looping, 1: looping, 0xFF: no animation.
-	uint8  speed;    ///< The speed, i.e. the amount of time between frames.
-	uint16 triggers; ///< The triggers that trigger animation.
+	uint8_t  frames;   ///< The number of frames.
+	uint8_t  status;   ///< Status; 0: no looping, 1: looping, 0xFF: no animation.
+	uint8_t  speed;    ///< The speed, i.e. the amount of time between frames.
+	uint16_t triggers; ///< The triggers that trigger animation.
 };
 
 /** Animation triggers for station. */
