@@ -45,17 +45,17 @@ static const IndustryType IT_AI_UNKNOWN = 0xFE; ///< The AI has no specific indu
 static const IndustryType IT_AI_TOWN    = 0xFF; ///< The AI actually wants to transport to/from a town, not an industry.
 
 void ResetGenericCallbacks();
-void AddGenericCallback(uint8 feature, const GRFFile *file, const SpriteGroup *group);
+void AddGenericCallback(uint8_t feature, const GRFFile *file, const SpriteGroup *group);
 
-uint16 GetAiPurchaseCallbackResult(uint8 feature, CargoID cargo_type, uint8 default_selection, IndustryType src_industry, IndustryType dst_industry, uint8 distance, AIConstructionEvent event, uint8 count, uint8 station_size, const GRFFile **file);
+uint16_t GetAiPurchaseCallbackResult(uint8_t feature, CargoID cargo_type, uint8_t default_selection, IndustryType src_industry, IndustryType dst_industry, uint8_t distance, AIConstructionEvent event, uint8_t count, uint8_t station_size, const GRFFile **file);
+void AmbientSoundEffectCallback(TileIndex tile);
 
 /** Play an ambient sound effect for an empty tile. */
-static inline void AmbientSoundEffect(TileIndex tile)
+inline void AmbientSoundEffect(TileIndex tile)
 {
 	/* Only run callback if enabled. */
 	if (!HasGrfMiscBit(GMB_AMBIENT_SOUND_CALLBACK)) return;
 
-	extern void AmbientSoundEffectCallback(TileIndex tile);
 	AmbientSoundEffectCallback(tile);
 }
 
