@@ -14,14 +14,14 @@
 #include "newgrf_spritegroup.h"
 #include "newgrf_town.h"
 #include "economy_func.h"
-#include "date_type.h"
+#include "timer/timer_game_calendar.h"
 #include "object_type.h"
 #include "newgrf_animation_type.h"
 #include "newgrf_class.h"
 #include "newgrf_commons.h"
 
 /** Various object behaviours. */
-enum ObjectFlags {
+enum ObjectFlags : uint16_t {
 	OBJECT_FLAG_NONE               =       0, ///< Just nothing.
 	OBJECT_FLAG_ONLY_IN_SCENEDIT   = 1 <<  0, ///< Object can only be constructed in the scenario editor.
 	OBJECT_FLAG_CANNOT_REMOVE      = 1 <<  1, ///< Object can not be removed.
@@ -40,12 +40,12 @@ enum ObjectFlags {
 };
 DECLARE_ENUM_AS_BIT_SET(ObjectFlags)
 
-static const uint8 OBJECT_SIZE_1X1 = 0x11; ///< The value of a NewGRF's size property when the object is 1x1 tiles: low nibble for X, high nibble for Y.
+static const uint8_t OBJECT_SIZE_1X1 = 0x11; ///< The value of a NewGRF's size property when the object is 1x1 tiles: low nibble for X, high nibble for Y.
 
 void ResetObjects();
 
 /** Class IDs for objects. */
-enum ObjectClassID {
+enum ObjectClassID : uint8_t {
 	OBJECT_CLASS_BEGIN   =    0, ///< The lowest valid value
 	OBJECT_CLASS_MAX     = 0xFF, ///< Maximum number of classes.
 	INVALID_OBJECT_CLASS = 0xFF, ///< Class for the less fortunate.
@@ -60,22 +60,27 @@ DECLARE_POSTFIX_INCREMENT(ObjectClassID)
 struct ObjectSpec {
 	/* 2 because of the "normal" and "buy" sprite stacks. */
 	GRFFilePropsBase<2> grf_prop; ///< Properties related the the grf file
+	AnimationInfo animation;      ///< Information about the animation.
 	ObjectClassID cls_id;         ///< The class to which this spec belongs.
 	StringID name;                ///< The name for this object.
 
-	uint8 climate;                ///< In which climates is this object available?
-	uint8 size;                   ///< The size of this objects; low nibble for X, high nibble for Y.
-	uint8 build_cost_multiplier;  ///< Build cost multiplier per tile.
-	uint8 clear_cost_multiplier;  ///< Clear cost multiplier per tile.
-	Date introduction_date;       ///< From when can this object be built.
-	Date end_of_life_date;        ///< When can't this object be built anymore.
+	uint8_t climate;                ///< In which climates is this object available?
+	uint8_t size;                   ///< The size of this objects; low nibble for X, high nibble for Y.
+	uint8_t build_cost_multiplier;  ///< Build cost multiplier per tile.
+	uint8_t clear_cost_multiplier;  ///< Clear cost multiplier per tile.
+	TimerGameCalendar::Date introduction_date; ///< From when can this object be built.
+	TimerGameCalendar::Date end_of_life_date;  ///< When can't this object be built anymore.
 	ObjectFlags flags;            ///< Flags/settings related to the object.
-	AnimationInfo animation;      ///< Information about the animation.
-	uint16 callback_mask;         ///< Bitmask of requested/allowed callbacks.
-	uint8 height;                 ///< The height of this structure, in heightlevels; max MAX_TILE_HEIGHT.
-	uint8 views;                  ///< The number of views.
-	uint8 generate_amount;        ///< Number of objects which are attempted to be generated per 256^2 map during world generation.
-	bool enabled;                 ///< Is this spec enabled?
+	uint16_t callback_mask;         ///< Bitmask of requested/allowed callbacks.
+	uint8_t height;                 ///< The height of this structure, in heightlevels; max MAX_TILE_HEIGHT.
+	uint8_t views;                  ///< The number of views.
+	uint8_t generate_amount;        ///< Number of objects which are attempted to be generated per 256^2 map during world generation.
+
+	/**
+	 * Test if this object is enabled.
+	 * @return True iff this object is enabled.
+	 */
+	bool IsEnabled() const { return this->views > 0; }
 
 	/**
 	 * Get the cost for building a structure of this type.
@@ -94,8 +99,12 @@ struct ObjectSpec {
 	bool IsAvailable() const;
 	uint Index() const;
 
+	static const std::vector<ObjectSpec> &Specs();
+	static size_t Count();
 	static const ObjectSpec *Get(ObjectType index);
 	static const ObjectSpec *GetByTile(TileIndex tile);
+
+	static void BindToClasses();
 };
 
 /** Object scope resolver. */
@@ -103,7 +112,7 @@ struct ObjectScopeResolver : public ScopeResolver {
 	struct Object *obj;     ///< The object the callback is ran for.
 	const ObjectSpec *spec; ///< Specification of the object type.
 	TileIndex tile;         ///< The tile related to the object.
-	uint8 view;             ///< The view of the object.
+	uint8_t view;             ///< The view of the object.
 
 	/**
 	 * Constructor of an object scope resolver.
@@ -112,13 +121,13 @@ struct ObjectScopeResolver : public ScopeResolver {
 	 * @param tile %Tile of the object.
 	 * @param view View of the object.
 	 */
-	ObjectScopeResolver(ResolverObject &ro, Object *obj, const ObjectSpec *spec, TileIndex tile, uint8 view = 0)
+	ObjectScopeResolver(ResolverObject &ro, Object *obj, const ObjectSpec *spec, TileIndex tile, uint8_t view = 0)
 		: ScopeResolver(ro), obj(obj), spec(spec), tile(tile), view(view)
 	{
 	}
 
-	uint32 GetRandomBits() const override;
-	uint32 GetVariable(byte variable, uint32 parameter, bool *available) const override;
+	uint32_t GetRandomBits() const override;
+	uint32_t GetVariable(byte variable, [[maybe_unused]] uint32_t parameter, bool *available) const override;
 };
 
 /** A resolver object to be used with feature 0F spritegroups. */
@@ -126,8 +135,8 @@ struct ObjectResolverObject : public ResolverObject {
 	ObjectScopeResolver object_scope; ///< The object scope resolver.
 	TownScopeResolver *town_scope;    ///< The town scope resolver (created on the first call).
 
-	ObjectResolverObject(const ObjectSpec *spec, Object *o, TileIndex tile, uint8 view = 0,
-			CallbackID callback = CBID_NO_CALLBACK, uint32 param1 = 0, uint32 param2 = 0);
+	ObjectResolverObject(const ObjectSpec *spec, Object *o, TileIndex tile, uint8_t view = 0,
+			CallbackID callback = CBID_NO_CALLBACK, uint32_t param1 = 0, uint32_t param2 = 0);
 	~ObjectResolverObject();
 
 	ScopeResolver *GetScope(VarSpriteGroupScope scope = VSG_SCOPE_SELF, byte relative = 0) override
@@ -139,7 +148,7 @@ struct ObjectResolverObject : public ResolverObject {
 			case VSG_SCOPE_PARENT: {
 				TownScopeResolver *tsr = this->GetTown();
 				if (tsr != nullptr) return tsr;
-				FALLTHROUGH;
+				[[fallthrough]];
 			}
 
 			default:
@@ -148,7 +157,7 @@ struct ObjectResolverObject : public ResolverObject {
 	}
 
 	GrfSpecFeature GetFeature() const override;
-	uint32 GetDebugID() const override;
+	uint32_t GetDebugID() const override;
 
 private:
 	TownScopeResolver *GetTown();
@@ -157,13 +166,13 @@ private:
 /** Struct containing information relating to object classes. */
 typedef NewGRFClass<ObjectSpec, ObjectClassID, OBJECT_CLASS_MAX> ObjectClass;
 
-/** Mapping of purchase for objects. */
-static const CargoID CT_PURCHASE_OBJECT = 1;
+static const size_t OBJECT_SPRITE_GROUP_DEFAULT = 0;
+static const size_t OBJECT_SPRITE_GROUP_PURCHASE = 1;
 
-uint16 GetObjectCallback(CallbackID callback, uint32 param1, uint32 param2, const ObjectSpec *spec, Object *o, TileIndex tile, uint8 view = 0);
+uint16_t GetObjectCallback(CallbackID callback, uint32_t param1, uint32_t param2, const ObjectSpec *spec, Object *o, TileIndex tile, uint8_t view = 0);
 
 void DrawNewObjectTile(TileInfo *ti, const ObjectSpec *spec);
-void DrawNewObjectTileInGUI(int x, int y, const ObjectSpec *spec, uint8 view);
+void DrawNewObjectTileInGUI(int x, int y, const ObjectSpec *spec, uint8_t view);
 void AnimateNewObjectTile(TileIndex tile);
 void TriggerObjectTileAnimation(Object *o, TileIndex tile, ObjectAnimationTrigger trigger, const ObjectSpec *spec);
 void TriggerObjectAnimation(Object *o, ObjectAnimationTrigger trigger, const ObjectSpec *spec);
