@@ -11,6 +11,7 @@
 #define OPENTTD_H
 
 #include <atomic>
+#include <chrono>
 #include "core/enum_type.hpp"
 
 /** Mode which defines the state of the game. */
@@ -51,8 +52,15 @@ enum DisplayOptions {
 	DO_SHOW_COMPETITOR_SIGNS = 7, ///< Display signs, station names and waypoint names of opponent companies. Buoys and oilrig-stations are always shown, even if this option is turned off.
 };
 
+struct GameSessionStats {
+	std::chrono::steady_clock::time_point start_time; ///< Time when the current game was started.
+	std::string savegame_id; ///< Unique ID of the savegame.
+	std::optional<size_t> savegame_size; ///< Size of the last saved savegame in bytes, or std::nullopt if not saved yet.
+};
+
 extern GameMode _game_mode;
 extern SwitchMode _switch_mode;
+extern GameSessionStats _game_session_stats;
 extern std::atomic<bool> _exit_game;
 extern bool _save_config;
 
@@ -66,6 +74,7 @@ enum PauseMode : byte {
 	PM_PAUSED_ACTIVE_CLIENTS = 1 << 4, ///< A game paused for 'min_active_clients'
 	PM_PAUSED_GAME_SCRIPT    = 1 << 5, ///< A game paused by a game script
 	PM_PAUSED_LINK_GRAPH     = 1 << 6, ///< A game paused due to the link graph schedule lagging
+	PM_COMMAND_DURING_PAUSE  = 1 << 7, ///< A game paused, and a command executed during the pause; resets on autosave
 
 	/** Pause mode bits when paused for network reasons. */
 	PMB_PAUSED_NETWORK = PM_PAUSED_ACTIVE_CLIENTS | PM_PAUSED_JOIN,
@@ -79,10 +88,15 @@ void AskExitGame();
 void AskExitToGameMenu();
 
 int openttd_main(int argc, char *argv[]);
+void StateGameLoop();
 void HandleExitGameRequest();
 
 void SwitchToMode(SwitchMode new_mode);
 
 bool RequestNewGRFScan(struct NewGRFScanCallback *callback = nullptr);
+void GenerateSavegameId();
+
+void OpenBrowser(const std::string &url);
+void ChangeAutosaveFrequency(bool reset);
 
 #endif /* OPENTTD_H */
