@@ -11,6 +11,7 @@
 #define PATHFINDER_TYPE_H
 
 #include "../tile_type.h"
+#include "npf/aystar.h"
 
 /** Length (penalty) of one tile with NPF */
 static const int NPF_TILE_LENGTH = 100;
@@ -38,9 +39,6 @@ static const int YAPF_TILE_CORNER_LENGTH = 71;
  */
 static const int YAPF_INFINITE_PENALTY = 1000 * YAPF_TILE_LENGTH;
 
-/** Maximum length of ship path cache */
-static const int YAPF_SHIP_PATH_CACHE_LENGTH = 32;
-
 /** Maximum segments of road vehicle path cache */
 static const int YAPF_ROADVEH_PATH_CACHE_SEGMENTS = 8;
 
