@@ -12,14 +12,8 @@
 
 #include "../../landscape.h"
 #include "../pathfinder_func.h"
-#include "../pf_performance_timer.hpp"
 #include "yapf.h"
 
-//#undef FORCEINLINE
-//#define inline inline
-
-#include "../../misc/blob.hpp"
-#include "../../misc/str.hpp"
 #include "../../misc/fixedsizearray.hpp"
 #include "../../misc/array.hpp"
 #include "../../misc/hashtable.hpp"
