@@ -87,14 +87,14 @@ public:
 
 protected:
 	/** to access inherited path finder */
-	inline Tpf& Yapf()
+	inline Tpf &Yapf()
 	{
 		return *static_cast<Tpf *>(this);
 	}
 
 public:
 	/** return current settings (can be custom - company based - but later) */
-	inline const YAPFSettings& PfGetSettings() const
+	inline const YAPFSettings &PfGetSettings() const
 	{
 		return *m_settings;
 	}
@@ -167,7 +167,7 @@ public:
 	 * Calls NodeList::CreateNewNode() - allocates new node that can be filled and used
 	 *  as argument for AddStartupNode() or AddNewNode()
 	 */
-	inline Node& CreateNewNode()
+	inline Node &CreateNewNode()
 	{
 		Node &node = *m_nodes.CreateNewNode();
 		return node;
@@ -192,7 +192,7 @@ public:
 	{
 		bool is_choice = (KillFirstBit(tf.m_new_td_bits) != TRACKDIR_BIT_NONE);
 		for (TrackdirBits rtds = tf.m_new_td_bits; rtds != TRACKDIR_BIT_NONE; rtds = KillFirstBit(rtds)) {
-			Trackdir td = (Trackdir)FindFirstBit2x64(rtds);
+			Trackdir td = (Trackdir)FindFirstBit(rtds);
 			Node &n = Yapf().CreateNewNode();
 			n.Set(parent, tf.m_new_tile, td, is_choice);
 			Yapf().AddNewNode(n, tf);
