@@ -13,7 +13,7 @@
 /** key for cached segment cost for rail YAPF */
 struct CYapfRailSegmentKey
 {
-	uint32    m_value;
+	uint32_t    m_value;
 
 	inline CYapfRailSegmentKey(const CYapfNodeKeyTrackDir &node_key)
 	{
@@ -27,10 +27,10 @@ struct CYapfRailSegmentKey
 
 	inline void Set(const CYapfNodeKeyTrackDir &node_key)
 	{
-		m_value = (((int)node_key.m_tile) << 4) | node_key.m_td;
+		m_value = (node_key.m_tile.base() << 4) | node_key.m_td;
 	}
 
-	inline int32 CalcHash() const
+	inline int32_t CalcHash() const
 	{
 		return m_value;
 	}
@@ -82,7 +82,7 @@ struct CYapfRailSegment
 		, m_hash_next(nullptr)
 	{}
 
-	inline const Key& GetKey() const
+	inline const Key &GetKey() const
 	{
 		return m_key;
 	}
@@ -123,13 +123,13 @@ struct CYapfRailNodeT
 	typedef CYapfRailSegment CachedData;
 
 	CYapfRailSegment *m_segment;
-	uint16            m_num_signals_passed;
+	uint16_t            m_num_signals_passed;
 	union {
-		uint32          m_inherited_flags;
+		uint32_t          m_inherited_flags;
 		struct {
-			bool          m_targed_seen : 1;
-			bool          m_choice_seen : 1;
-			bool          m_last_signal_was_red : 1;
+			bool          m_targed_seen;
+			bool          m_choice_seen;
+			bool          m_last_signal_was_red;
 		} flags_s;
 	} flags_u;
 	SignalType        m_last_red_signal_type;
@@ -142,7 +142,7 @@ struct CYapfRailNodeT
 		if (parent == nullptr) {
 			m_num_signals_passed      = 0;
 			flags_u.m_inherited_flags = 0;
-			m_last_red_signal_type    = SIGTYPE_NORMAL;
+			m_last_red_signal_type    = SIGTYPE_BLOCK;
 			/* We use PBS as initial signal type because if we are in
 			 * a PBS section and need to route, i.e. we're at a safe
 			 * waiting point of a station, we need to account for the
