@@ -17,19 +17,22 @@
 #include "economy_func.h"
 #include "slope_type.h"
 #include "strings_type.h"
-#include "date_type.h"
+#include "timer/timer_game_calendar.h"
 #include "signal_type.h"
 #include "settings_type.h"
 
-/** Railtype flags. */
-enum RailTypeFlags {
+/** Railtype flag bit numbers. */
+enum RailTypeFlag {
 	RTF_CATENARY          = 0,                           ///< Bit number for drawing a catenary.
 	RTF_NO_LEVEL_CROSSING = 1,                           ///< Bit number for disallowing level crossings.
 	RTF_HIDDEN            = 2,                           ///< Bit number for hiding from selection.
 	RTF_NO_SPRITE_COMBINE = 3,                           ///< Bit number for using non-combined junctions.
 	RTF_ALLOW_90DEG       = 4,                           ///< Bit number for always allowed 90 degree turns, regardless of setting.
 	RTF_DISALLOW_90DEG    = 5,                           ///< Bit number for never allowed 90 degree turns, regardless of setting.
+};
 
+/** Railtype flags. */
+enum RailTypeFlags : uint8_t {
 	RTFB_NONE              = 0,                          ///< All flags cleared.
 	RTFB_CATENARY          = 1 << RTF_CATENARY,          ///< Value for drawing a catenary.
 	RTFB_NO_LEVEL_CROSSING = 1 << RTF_NO_LEVEL_CROSSING, ///< Value for disallowing level crossings.
@@ -121,7 +124,7 @@ typedef std::vector<RailTypeLabel> RailTypeLabelList;
 /**
  * This struct contains all the info that is needed to draw and construct tracks.
  */
-class RailtypeInfo {
+class RailTypeInfo {
 public:
 	/**
 	 * Struct containing the main sprites. @note not all sprites are listed, but only
@@ -210,22 +213,22 @@ public:
 	/**
 	 * Cost multiplier for building this rail type
 	 */
-	uint16 cost_multiplier;
+	uint16_t cost_multiplier;
 
 	/**
 	 * Cost multiplier for maintenance of this rail type
 	 */
-	uint16 maintenance_multiplier;
+	uint16_t maintenance_multiplier;
 
 	/**
 	 * Acceleration type of this rail type
 	 */
-	uint8 acceleration_type;
+	uint8_t acceleration_type;
 
 	/**
 	 * Maximum speed for vehicles travelling on this rail type
 	 */
-	uint16 max_speed;
+	uint16_t max_speed;
 
 	/**
 	 * Unique 32 bit rail type identifier
@@ -249,7 +252,7 @@ public:
 	 * The introduction at this date is furthermore limited by the
 	 * #introduction_required_railtypes.
 	 */
-	Date introduction_date;
+	TimerGameCalendar::Date introduction_date;
 
 	/**
 	 * Bitmask of railtypes that are required for this railtype to be introduced
@@ -299,11 +302,11 @@ public:
 /**
  * Returns a pointer to the Railtype information for a given railtype
  * @param railtype the rail type which the information is requested for
- * @return The pointer to the RailtypeInfo
+ * @return The pointer to the RailTypeInfo
  */
-static inline const RailtypeInfo *GetRailTypeInfo(RailType railtype)
+inline const RailTypeInfo *GetRailTypeInfo(RailType railtype)
 {
-	extern RailtypeInfo _railtypes[RAILTYPE_END];
+	extern RailTypeInfo _railtypes[RAILTYPE_END];
 	assert(railtype < RAILTYPE_END);
 	return &_railtypes[railtype];
 }
@@ -316,7 +319,7 @@ static inline const RailtypeInfo *GetRailTypeInfo(RailType railtype)
  * @param  enginetype The RailType of the engine we are considering.
  * @param  tiletype   The RailType of the tile we are considering.
  */
-static inline bool IsCompatibleRail(RailType enginetype, RailType tiletype)
+inline bool IsCompatibleRail(RailType enginetype, RailType tiletype)
 {
 	return HasBit(GetRailTypeInfo(enginetype)->compatible_railtypes, tiletype);
 }
@@ -329,7 +332,7 @@ static inline bool IsCompatibleRail(RailType enginetype, RailType tiletype)
  * @param  enginetype The RailType of the engine we are considering.
  * @param  tiletype   The RailType of the tile we are considering.
  */
-static inline bool HasPowerOnRail(RailType enginetype, RailType tiletype)
+inline bool HasPowerOnRail(RailType enginetype, RailType tiletype)
 {
 	return HasBit(GetRailTypeInfo(enginetype)->powered_railtypes, tiletype);
 }
@@ -339,7 +342,7 @@ static inline bool HasPowerOnRail(RailType enginetype, RailType tiletype)
  * @param rt The RailType to check.
  * @return Whether level crossings are not allowed.
  */
-static inline bool RailNoLevelCrossings(RailType rt)
+inline bool RailNoLevelCrossings(RailType rt)
 {
 	return HasBit(GetRailTypeInfo(rt)->flags, RTF_NO_LEVEL_CROSSING);
 }
@@ -351,12 +354,12 @@ static inline bool RailNoLevelCrossings(RailType rt)
  * @param def Default value to use if the rail type doesn't specify anything.
  * @return True if 90 degree turns are disallowed between the two rail types.
  */
-static inline bool Rail90DegTurnDisallowed(RailType rt1, RailType rt2, bool def = _settings_game.pf.forbid_90_deg)
+inline bool Rail90DegTurnDisallowed(RailType rt1, RailType rt2, bool def = _settings_game.pf.forbid_90_deg)
 {
 	if (rt1 == INVALID_RAILTYPE || rt2 == INVALID_RAILTYPE) return def;
 
-	const RailtypeInfo *rti1 = GetRailTypeInfo(rt1);
-	const RailtypeInfo *rti2 = GetRailTypeInfo(rt2);
+	const RailTypeInfo *rti1 = GetRailTypeInfo(rt1);
+	const RailTypeInfo *rti2 = GetRailTypeInfo(rt2);
 
 	bool rt1_90deg = HasBit(rti1->flags, RTF_DISALLOW_90DEG) || (!HasBit(rti1->flags, RTF_ALLOW_90DEG) && def);
 	bool rt2_90deg = HasBit(rti2->flags, RTF_DISALLOW_90DEG) || (!HasBit(rti2->flags, RTF_ALLOW_90DEG) && def);
@@ -369,7 +372,7 @@ static inline bool Rail90DegTurnDisallowed(RailType rt1, RailType rt2, bool def
  * @param railtype The railtype being built.
  * @return The cost multiplier.
  */
-static inline Money RailBuildCost(RailType railtype)
+inline Money RailBuildCost(RailType railtype)
 {
 	assert(railtype < RAILTYPE_END);
 	return (_price[PR_BUILD_RAIL] * GetRailTypeInfo(railtype)->cost_multiplier) >> 3;
@@ -380,7 +383,7 @@ static inline Money RailBuildCost(RailType railtype)
  * @param railtype The railtype being removed.
  * @return The cost.
  */
-static inline Money RailClearCost(RailType railtype)
+inline Money RailClearCost(RailType railtype)
 {
 	/* Clearing rail in fact earns money, but if the build cost is set
 	 * very low then a loophole exists where money can be made.
@@ -397,7 +400,7 @@ static inline Money RailClearCost(RailType railtype)
  * @param to   The railtype we are converting to
  * @return Cost per TrackBit
  */
-static inline Money RailConvertCost(RailType from, RailType to)
+inline Money RailConvertCost(RailType from, RailType to)
 {
 	/* Get the costs for removing and building anew
 	 * A conversion can never be more costly */
@@ -424,7 +427,7 @@ static inline Money RailConvertCost(RailType from, RailType to)
  * @param total_num Total number of track bits of all railtypes.
  * @return Total cost.
  */
-static inline Money RailMaintenanceCost(RailType railtype, uint32 num, uint32 total_num)
+inline Money RailMaintenanceCost(RailType railtype, uint32_t num, uint32_t total_num)
 {
 	assert(railtype < RAILTYPE_END);
 	return (_price[PR_INFRASTRUCTURE_RAIL] * GetRailTypeInfo(railtype)->maintenance_multiplier * num * (1 + IntSqrt(total_num))) >> 11; // 4 bits fraction for the multiplier and 7 bits scaling.
@@ -435,7 +438,7 @@ static inline Money RailMaintenanceCost(RailType railtype, uint32 num, uint32 to
  * @param num Number of signals.
  * @return Total cost.
  */
-static inline Money SignalMaintenanceCost(uint32 num)
+inline Money SignalMaintenanceCost(uint32_t num)
 {
 	return (_price[PR_INFRASTRUCTURE_RAIL] * 15 * num * (1 + IntSqrt(num))) >> 8; // 1 bit fraction for the multiplier and 7 bits scaling.
 }
@@ -446,13 +449,13 @@ int TicksToLeaveDepot(const Train *v);
 Foundation GetRailFoundation(Slope tileh, TrackBits bits);
 
 
-bool HasRailtypeAvail(const CompanyID company, const RailType railtype);
-bool HasAnyRailtypesAvail(const CompanyID company);
-bool ValParamRailtype(const RailType rail);
+bool HasRailTypeAvail(const CompanyID company, const RailType railtype);
+bool HasAnyRailTypesAvail(const CompanyID company);
+bool ValParamRailType(const RailType rail);
 
-RailTypes AddDateIntroducedRailTypes(RailTypes current, Date date);
+RailTypes AddDateIntroducedRailTypes(RailTypes current, TimerGameCalendar::Date date);
 
-RailTypes GetCompanyRailtypes(CompanyID company, bool introduces = true);
+RailTypes GetCompanyRailTypes(CompanyID company, bool introduces = true);
 RailTypes GetRailTypes(bool introduces);
 
 RailType GetRailTypeByLabel(RailTypeLabel label, bool allow_alternate_labels = true);
