@@ -12,7 +12,10 @@
 
 #include "core/enum_type.hpp"
 
-typedef uint32 RoadTypeLabel;
+typedef uint32_t RoadTypeLabel;
+
+static const RoadTypeLabel ROADTYPE_LABEL_ROAD = 'ROAD';
+static const RoadTypeLabel ROADTYPE_LABEL_TRAM = 'ELRL';
 
 /**
  * The different roadtypes we support
@@ -27,13 +30,12 @@ enum RoadType : byte {
 	INVALID_ROADTYPE = 63,   ///< flag for invalid roadtype
 };
 DECLARE_POSTFIX_INCREMENT(RoadType)
-template <> struct EnumPropsT<RoadType> : MakeEnumPropsT<RoadType, byte, ROADTYPE_BEGIN, ROADTYPE_END, INVALID_ROADTYPE, 6> {};
 
 /**
  * The different roadtypes we support, but then a bitmask of them.
- * @note Must be treated as a uint64 type, narrowing it causes bit membership tests to give wrong results.
+ * @note Must be treated as a uint64_t type, narrowing it causes bit membership tests to give wrong results.
  */
-enum RoadTypes : uint64 {
+enum RoadTypes : uint64_t {
 	ROADTYPES_NONE     = 0,                                ///< No roadtypes
 	ROADTYPES_ROAD     = 1 << ROADTYPE_ROAD,               ///< Road
 	ROADTYPES_TRAM     = 1 << ROADTYPE_TRAM,               ///< Trams
@@ -66,7 +68,6 @@ enum RoadBits : byte {
 	ROAD_END  = ROAD_ALL + 1,        ///< Out-of-range roadbits, used for iterations
 };
 DECLARE_ENUM_AS_BIT_SET(RoadBits)
-template <> struct EnumPropsT<RoadBits> : MakeEnumPropsT<RoadBits, byte, ROAD_NONE, ROAD_END, ROAD_NONE, 4> {};
 
 /** Which directions are disallowed ? */
 enum DisallowedRoadDirections : byte {
@@ -77,7 +78,5 @@ enum DisallowedRoadDirections : byte {
 	DRD_END,        ///< Sentinel
 };
 DECLARE_ENUM_AS_BIT_SET(DisallowedRoadDirections)
-/** Helper information for extract tool. */
-template <> struct EnumPropsT<DisallowedRoadDirections> : MakeEnumPropsT<DisallowedRoadDirections, byte, DRD_NONE, DRD_END, DRD_END, 2> {};
 
 #endif /* ROAD_TYPE_H */
