@@ -8,10 +8,12 @@
 /** @file object_sl.cpp Code handling saving and loading of objects */
 
 #include "../stdafx.h"
-#include "../object_base.h"
-#include "../object_map.h"
 
 #include "saveload.h"
+#include "compat/object_sl_compat.h"
+
+#include "../object_base.h"
+#include "../object_map.h"
 #include "newgrf_sl.h"
 
 #include "../safeguards.h"
@@ -25,50 +27,54 @@ static const SaveLoad _object_desc[] = {
 	SLE_CONDVAR(Object, colour,                     SLE_UINT8,                  SLV_148, SL_MAX_VERSION),
 	SLE_CONDVAR(Object, view,                       SLE_UINT8,                  SLV_155, SL_MAX_VERSION),
 	SLE_CONDVAR(Object, type,                       SLE_UINT16,                 SLV_186, SL_MAX_VERSION),
-
-	SLE_END()
 };
 
-static void Save_OBJS()
-{
-	/* Write the objects */
-	for (Object *o : Object::Iterate()) {
-		SlSetArrayIndex(o->index);
-		SlObject(o, _object_desc);
-	}
-}
+struct OBJSChunkHandler : ChunkHandler {
+	OBJSChunkHandler() : ChunkHandler('OBJS', CH_TABLE) {}
 
-static void Load_OBJS()
-{
-	int index;
-	while ((index = SlIterateArray()) != -1) {
-		Object *o = new (index) Object();
-		SlObject(o, _object_desc);
-	}
-}
+	void Save() const override
+	{
+		SlTableHeader(_object_desc);
 
-static void Ptrs_OBJS()
-{
-	for (Object *o : Object::Iterate()) {
-		SlObject(o, _object_desc);
-		if (IsSavegameVersionBefore(SLV_148) && !IsTileType(o->location.tile, MP_OBJECT)) {
-			/* Due to a small bug stale objects could remain. */
-			delete o;
+		/* Write the objects */
+		for (Object *o : Object::Iterate()) {
+			SlSetArrayIndex(o->index);
+			SlObject(o, _object_desc);
 		}
 	}
-}
 
-static void Save_OBID()
-{
-	Save_NewGRFMapping(_object_mngr);
-}
+	void Load() const override
+	{
+		const std::vector<SaveLoad> slt = SlCompatTableHeader(_object_desc, _object_sl_compat);
 
-static void Load_OBID()
-{
-	Load_NewGRFMapping(_object_mngr);
-}
+		int index;
+		while ((index = SlIterateArray()) != -1) {
+			Object *o = new (index) Object();
+			SlObject(o, slt);
+		}
+	}
 
-extern const ChunkHandler _object_chunk_handlers[] = {
-	{ 'OBID', Save_OBID, Load_OBID, nullptr,   nullptr, CH_ARRAY },
-	{ 'OBJS', Save_OBJS, Load_OBJS, Ptrs_OBJS, nullptr, CH_ARRAY | CH_LAST},
+	void FixPointers() const override
+	{
+		for (Object *o : Object::Iterate()) {
+			SlObject(o, _object_desc);
+			if (IsSavegameVersionBefore(SLV_148) && !IsTileType(o->location.tile, MP_OBJECT)) {
+				/* Due to a small bug stale objects could remain. */
+				delete o;
+			}
+		}
+	}
 };
+
+struct OBIDChunkHandler : NewGRFMappingChunkHandler {
+	OBIDChunkHandler() : NewGRFMappingChunkHandler('OBID', _object_mngr) {}
+};
+
+static const OBIDChunkHandler OBID;
+static const OBJSChunkHandler OBJS;
+static const ChunkHandlerRef object_chunk_handlers[] = {
+	OBID,
+	OBJS,
+};
+
+extern const ChunkHandlerTable _object_chunk_handlers(object_chunk_handlers);
