@@ -42,12 +42,12 @@ public:
 	 */
 	enum TownEffect {
 		/* Note: these values represent part of the in-game TownEffect enum */
-		TE_NONE       = ::TE_NONE,       ///< This cargo has no effect on a town
-		TE_PASSENGERS = ::TE_PASSENGERS, ///< This cargo supplies passengers to a town
-		TE_MAIL       = ::TE_MAIL,       ///< This cargo supplies mail to a town
-		TE_GOODS      = ::TE_GOODS,      ///< This cargo supplies goods to a town
-		TE_WATER      = ::TE_WATER,      ///< This cargo supplies water to a town
-		TE_FOOD       = ::TE_FOOD,       ///< This cargo supplies food to a town
+		TE_NONE       = ::TAE_NONE,       ///< This cargo has no effect on a town
+		TE_PASSENGERS = ::TAE_PASSENGERS, ///< This cargo supplies passengers to a town
+		TE_MAIL       = ::TAE_MAIL,       ///< This cargo supplies mail to a town
+		TE_GOODS      = ::TAE_GOODS,      ///< This cargo supplies goods to a town
+		TE_WATER      = ::TAE_WATER,      ///< This cargo supplies water to a town
+		TE_FOOD       = ::TAE_FOOD,       ///< This cargo supplies food to a town
 	};
 
 	/**
@@ -55,9 +55,9 @@ public:
 	 */
 	enum SpecialCargoID {
 		/* Note: these values represent part of the in-game CargoTypes enum */
-		CT_AUTO_REFIT = ::CT_AUTO_REFIT, ///< Automatically choose cargo type when doing auto-refitting.
-		CT_NO_REFIT   = ::CT_NO_REFIT,   ///< Do not refit cargo of a vehicle.
-		CT_INVALID    = ::CT_INVALID,    ///< An invalid cargo type.
+		CT_AUTO_REFIT = ::CARGO_AUTO_REFIT, ///< Automatically choose cargo type when doing auto-refitting.
+		CT_NO_REFIT   = ::CARGO_NO_REFIT, ///< Do not refit cargo of a vehicle.
+		CT_INVALID    = ::INVALID_CARGO, ///< An invalid cargo type.
 	};
 
 	/**
@@ -90,7 +90,7 @@ public:
 	 * @pre IsValidCargo(cargo_type).
 	 * @return The name of the cargo type.
 	 */
-	static char *GetName(CargoID cargo_type);
+	static std::optional<std::string> GetName(CargoID cargo_type);
 
 	/**
 	 * Gets the string representation of the cargo label.
@@ -107,7 +107,7 @@ public:
 	 *  - In other words: Only use the cargo label, if you know more about the behaviour
 	 *    of a specific cargo from a specific industry set, than the API methods can tell you.
 	 */
-	static char *GetCargoLabel(CargoID cargo_type);
+	static std::optional<std::string> GetCargoLabel(CargoID cargo_type);
 
 	/**
 	 * Checks whether the give cargo is a freight or not.
@@ -142,10 +142,12 @@ public:
 	 * @param cargo_type The cargo to transport.
 	 * @pre ScriptCargo::IsValidCargo(cargo_type).
 	 * @param distance The distance the cargo travels from begin to end.
-	 * @param days_in_transit Amount of (game) days the cargo is in transit. The max value of this variable is 637. Any value higher returns the same as 637 would.
+	 *                 The value will be clamped to 0 .. MAX(uint32_t).
+	 * @param days_in_transit Amount of (game) days the cargo is in transit.
+	 *                        The max value of this variable is 637. Any value higher returns the same as 637 would.
 	 * @return The amount of money that would be earned by this trip.
 	 */
-	static Money GetCargoIncome(CargoID cargo_type, uint32 distance, uint32 days_in_transit);
+	static Money GetCargoIncome(CargoID cargo_type, SQInteger distance, SQInteger days_in_transit);
 
 	/**
 	 * Get the cargo distribution type for a cargo.
@@ -159,10 +161,11 @@ public:
 	 *   cargo for the specified type.
 	 * @param cargo_type The cargo to check on.
 	 * @param amount The quantity of cargo.
+	 *               The value will be clamped to 0 .. MAX(uint32_t).
 	 * @pre ScriptCargo::IsValidCargo(cargo_type).
 	 * @return The weight in tonnes for that quantity of cargo.
 	 */
-	static int64 GetWeight(CargoID cargo_type, uint32 amount);
+	static SQInteger GetWeight(CargoID cargo_type, SQInteger amount);
 };
 
 #endif /* SCRIPT_CARGO_HPP */
