@@ -129,8 +129,8 @@ public:
 	/**
 	 * Check if a CompanyID is your CompanyID, to ease up checks.
 	 * @param company The company index to check.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if and only if this company is your CompanyID.
-	 * @api -game
 	 */
 	static bool IsMine(CompanyID company);
 
@@ -138,6 +138,7 @@ public:
 	 * Set the name of your company.
 	 * @param name The new name of the company (can be either a raw string, or a ScriptText object).
 	 * @pre name != null && len(name) != 0.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @exception ScriptError::ERR_NAME_IS_NOT_UNIQUE
 	 * @return True if the name was changed.
 	 */
@@ -149,12 +150,13 @@ public:
 	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
 	 * @return The name of the given company.
 	 */
-	static char *GetName(CompanyID company);
+	static std::optional<std::string> GetName(CompanyID company);
 
 	/**
 	 * Set the name of your president.
 	 * @param name The new name of the president (can be either a raw string, or a ScriptText object).
 	 * @pre name != null && len(name) != 0.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @exception ScriptError::ERR_NAME_IS_NOT_UNIQUE
 	 * @return True if the name was changed.
 	 */
@@ -166,15 +168,15 @@ public:
 	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
 	 * @return The name of the president of the given company.
 	 */
-	static char *GetPresidentName(CompanyID company);
+	static std::optional<std::string> GetPresidentName(CompanyID company);
 
 	/**
 	 * Set the gender of the president of your company.
 	 * @param gender The new gender for your president.
 	 * @pre GetPresidentGender(ScriptCompany.COMPANY_SELF) != gender.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if the gender was changed.
 	 * @note When successful a random face will be created.
-	 * @api -game
 	 */
 	static bool SetPresidentGender(Gender gender);
 
@@ -192,7 +194,7 @@ public:
 	 * @pre GetLoanInterval() must be a multiplier of 'loan'.
 	 * @pre 'loan' must be below GetMaxLoanAmount().
 	 * @pre 'loan' - GetLoanAmount() + GetBankBalance() must be non-negative.
-	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if the loan could be set to your requested amount.
 	 */
 	static bool SetLoanAmount(Money loan);
@@ -202,7 +204,7 @@ public:
 	 * @param loan The amount to loan (any positive number).
 	 * @pre 'loan' must be non-negative.
 	 * @pre 'loan' must be below GetMaxLoanAmount().
-	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if we could allocate a minimum of 'loan' loan.
 	 */
 	static bool SetMinimumLoanAmount(Money loan);
@@ -215,12 +217,38 @@ public:
 	static Money GetLoanAmount();
 
 	/**
-	 * Gets the maximum amount your company can loan.
+	 * Gets the maximum amount your company can loan. In deity mode returns the global max loan.
 	 * @return The maximum amount your company can loan.
 	 * @post GetLoanInterval() is always a multiplier of the return value.
 	 */
 	static Money GetMaxLoanAmount();
 
+	/**
+	 * Sets the max amount of money company can loan.
+	 * @param company The company ID.
+	 * @param amount Max loan amount. Will be rounded down to a multiple of GetLoanInterval().
+	 * @return True, if the max loan was changed.
+	 * @pre ScriptCompanyMode::IsDeity().
+	 * @pre amount >= 0.
+	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
+	 * @note You need to create your own news message to inform about max loan change.
+	 * @note Max loan value set with this method is not affected by inflation.
+	 * @api -ai
+	 */
+	static bool SetMaxLoanAmountForCompany(CompanyID company, Money amount);
+
+	/**
+	 * Makes the max amount of money company can loan follow the global max loan setting.
+	 * @param company The company ID.
+	 * @return True, if the max loan was reset.
+	 * @pre ScriptCompanyMode::IsDeity().
+	 * @pre amount >= 0 && amount <= MAX_LOAN_LIMIT.
+	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
+	 * @note You need to create your own news message to inform about max loan change.
+	 * @api -ai
+	 */
+	static bool ResetMaxLoanAmountForCompany(CompanyID company);
+
 	/**
 	 * Gets the interval/loan step.
 	 * @return The loan step.
@@ -230,9 +258,10 @@ public:
 
 	/**
 	 * Gets the bank balance. In other words, the amount of money the given company can spent.
+	 * If infinite money is enabled, it returns INT32_MAX.
 	 * @param company The company to get the bank balance of.
 	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
-	 * @return The actual bank balance.
+	 * @return The actual bank balance or INT32_MAX.
 	 */
 	static Money GetBankBalance(CompanyID company);
 
@@ -244,10 +273,8 @@ public:
 	 * @param expenses_type The account in the finances window that will register the cost.
 	 * @param tile The tile to show text effect on or ScriptMap::TILE_INVALID
 	 * @return True, if the bank balance was changed.
-	 * @game @pre No ScriptCompanyMode active in scope.
+	 * @game @pre ScriptCompanyMode::IsDeity().
 	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
-	 * @pre delta >= -2**31
-	 * @pre delta <   2**31
 	 * @note You need to create your own news message to inform about costs/gifts that you create using this command.
 	 * @api -ai
 	 */
@@ -263,7 +290,7 @@ public:
 	 * @pre quarter <= EARLIEST_QUARTER.
 	 * @return The gross income of the company in the given quarter.
 	 */
-	static Money GetQuarterlyIncome(CompanyID company, uint32 quarter);
+	static Money GetQuarterlyIncome(CompanyID company, SQInteger quarter);
 
 	/**
 	 * Get the expenses of the company in the given quarter.
@@ -276,7 +303,7 @@ public:
 	 * @pre quarter <= EARLIEST_QUARTER.
 	 * @return The expenses of the company in the given quarter.
 	 */
-	static Money GetQuarterlyExpenses(CompanyID company, uint32 quarter);
+	static Money GetQuarterlyExpenses(CompanyID company, SQInteger quarter);
 
 	/**
 	 * Get the amount of cargo delivered by the given company in the given quarter.
@@ -286,7 +313,7 @@ public:
 	 * @pre quarter <= EARLIEST_QUARTER.
 	 * @return The amount of cargo delivered by the given company in the given quarter.
 	 */
-	static int32 GetQuarterlyCargoDelivered(CompanyID company, uint32 quarter);
+	static SQInteger GetQuarterlyCargoDelivered(CompanyID company, SQInteger quarter);
 
 	/**
 	 * Get the performance rating of the given company in the given quarter.
@@ -298,7 +325,7 @@ public:
 	 * @note The performance rating is calculated after every quarter, so the value for CURRENT_QUARTER is undefined.
 	 * @return The performance rating of the given company in the given quarter.
 	 */
-	static int32 GetQuarterlyPerformanceRating(CompanyID company, uint32 quarter);
+	static SQInteger GetQuarterlyPerformanceRating(CompanyID company, SQInteger quarter);
 
 	/**
 	 * Get the value of the company in the given quarter.
@@ -308,13 +335,13 @@ public:
 	 * @pre quarter <= EARLIEST_QUARTER.
 	 * @return The value of the company in the given quarter.
 	 */
-	static Money GetQuarterlyCompanyValue(CompanyID company, uint32 quarter);
+	static Money GetQuarterlyCompanyValue(CompanyID company, SQInteger quarter);
 
 	/**
 	 * Build your company's HQ on the given tile.
 	 * @param tile The tile to build your HQ on, this tile is the most northern tile of your HQ.
 	 * @pre ScriptMap::IsValidTile(tile).
-	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @exception ScriptError::ERR_AREA_NOT_CLEAR
 	 * @exception ScriptError::ERR_FLAT_LAND_REQUIRED
 	 * @return True if the HQ could be build.
@@ -335,8 +362,8 @@ public:
 	/**
 	 * Set whether autorenew is enabled for your company.
 	 * @param autorenew The new autorenew status.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if autorenew status has been modified.
-	 * @api -game
 	 */
 	static bool SetAutoRenewStatus(bool autorenew);
 
@@ -351,10 +378,11 @@ public:
 	/**
 	 * Set the number of months before/after max age to autorenew an engine for your company.
 	 * @param months The new months between autorenew.
+	 *               The value will be clamped to MIN(int16_t) .. MAX(int16_t).
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if autorenew months has been modified.
-	 * @api -game
 	 */
-	static bool SetAutoRenewMonths(int16 months);
+	static bool SetAutoRenewMonths(SQInteger months);
 
 	/**
 	 * Return the number of months before/after max age to autorenew an engine for a company.
@@ -362,15 +390,15 @@ public:
 	 * @pre ResolveCompanyID(company) != COMPANY_INVALID.
 	 * @return The months before/after max age of engine.
 	 */
-	static int16 GetAutoRenewMonths(CompanyID company);
+	static SQInteger GetAutoRenewMonths(CompanyID company);
 
 	/**
 	 * Set the minimum money needed to autorenew an engine for your company.
 	 * @param money The new minimum required money for autorenew to work.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return True if autorenew money has been modified.
 	 * @pre money >= 0
 	 * @pre money <  2**32
-	 * @api -game
 	 */
 	static bool SetAutoRenewMoney(Money money);
 
@@ -386,6 +414,7 @@ public:
 	 * Set primary colour for your company.
 	 * @param scheme Livery scheme to set.
 	 * @param colour Colour to set.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return False if unable to set primary colour of the livery scheme (e.g. colour in use).
 	 */
 	static bool SetPrimaryLiveryColour(LiveryScheme scheme, Colours colour);
@@ -394,6 +423,7 @@ public:
 	 * Set secondary colour for your company.
 	 * @param scheme Livery scheme to set.
 	 * @param colour Colour to set.
+	 * @game @pre ScriptCompanyMode::IsValid().
 	 * @return False if unable to set secondary colour of the livery scheme.
 	 */
 	static bool SetSecondaryLiveryColour(LiveryScheme scheme, Colours colour);
