@@ -18,30 +18,33 @@
 #include "../../engine_func.h"
 #include "../../articulated_vehicles.h"
 #include "../../engine_cmd.h"
+#include "../../timer/timer_game_calendar.h"
 #include "table/strings.h"
 
 #include "../../safeguards.h"
 
 /* static */ bool ScriptEngine::IsValidEngine(EngineID engine_id)
 {
+	EnforceDeityOrCompanyModeValid(false);
 	const Engine *e = ::Engine::GetIfValid(engine_id);
 	if (e == nullptr || !e->IsEnabled()) return false;
 
 	/* AIs have only access to engines they can purchase or still have in use.
 	 * Deity has access to all engined that will be or were available ever. */
 	CompanyID company = ScriptObject::GetCompany();
-	return company == OWNER_DEITY || ::IsEngineBuildable(engine_id, e->type, company) || ::Company::Get(company)->group_all[e->type].num_engines[engine_id] > 0;
+	return ScriptCompanyMode::IsDeity() || ::IsEngineBuildable(engine_id, e->type, company) || ::Company::Get(company)->group_all[e->type].GetNumEngines(engine_id) > 0;
 }
 
 /* static */ bool ScriptEngine::IsBuildable(EngineID engine_id)
 {
+	EnforceDeityOrCompanyModeValid(false);
 	const Engine *e = ::Engine::GetIfValid(engine_id);
 	return e != nullptr && ::IsEngineBuildable(engine_id, e->type, ScriptObject::GetCompany());
 }
 
-/* static */ char *ScriptEngine::GetName(EngineID engine_id)
+/* static */ std::optional<std::string> ScriptEngine::GetName(EngineID engine_id)
 {
-	if (!IsValidEngine(engine_id)) return nullptr;
+	if (!IsValidEngine(engine_id)) return std::nullopt;
 
 	::SetDParam(0, engine_id);
 	return GetString(STR_ENGINE_NAME);
@@ -49,20 +52,14 @@
 
 /* static */ CargoID ScriptEngine::GetCargoType(EngineID engine_id)
 {
-	if (!IsValidEngine(engine_id)) return CT_INVALID;
+	if (!IsValidEngine(engine_id)) return INVALID_CARGO;
 
 	CargoArray cap = ::GetCapacityOfArticulatedParts(engine_id);
 
-	CargoID most_cargo = CT_INVALID;
-	uint amount = 0;
-	for (CargoID cid = 0; cid < NUM_CARGO; cid++) {
-		if (cap[cid] > amount) {
-			amount = cap[cid];
-			most_cargo = cid;
-		}
-	}
+	auto it = std::max_element(std::cbegin(cap), std::cend(cap));
+	if (*it == 0) return INVALID_CARGO;
 
-	return most_cargo;
+	return CargoID(std::distance(std::cbegin(cap), it));
 }
 
 /* static */ bool ScriptEngine::CanRefitCargo(EngineID engine_id, CargoID cargo_id)
@@ -83,7 +80,7 @@
 }
 
 
-/* static */ int32 ScriptEngine::GetCapacity(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetCapacity(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 
@@ -92,9 +89,8 @@
 		case VEH_ROAD:
 		case VEH_TRAIN: {
 			CargoArray capacities = GetCapacityOfArticulatedParts(engine_id);
-			for (CargoID c = 0; c < NUM_CARGO; c++) {
-				if (capacities[c] == 0) continue;
-				return capacities[c];
+			for (uint &cap : capacities) {
+				if (cap != 0) return cap;
 			}
 			return -1;
 		}
@@ -107,7 +103,7 @@
 	}
 }
 
-/* static */ int32 ScriptEngine::GetReliability(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetReliability(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 	if (GetVehicleType(engine_id) == ScriptVehicle::VT_RAIL && IsWagon(engine_id)) return -1;
@@ -115,12 +111,12 @@
 	return ::ToPercent16(::Engine::Get(engine_id)->reliability);
 }
 
-/* static */ int32 ScriptEngine::GetMaxSpeed(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetMaxSpeed(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 
 	const Engine *e = ::Engine::Get(engine_id);
-	int32 max_speed = e->GetDisplayMaxSpeed(); // km-ish/h
+	uint max_speed = e->GetDisplayMaxSpeed(); // km-ish/h
 	if (e->type == VEH_AIRCRAFT) max_speed /= _settings_game.vehicle.plane_speed;
 	return max_speed;
 }
@@ -132,12 +128,12 @@
 	return ::Engine::Get(engine_id)->GetCost();
 }
 
-/* static */ int32 ScriptEngine::GetMaxAge(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetMaxAge(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 	if (GetVehicleType(engine_id) == ScriptVehicle::VT_RAIL && IsWagon(engine_id)) return -1;
 
-	return ::Engine::Get(engine_id)->GetLifeLengthInDays();
+	return ::Engine::Get(engine_id)->GetLifeLengthInDays().base();
 }
 
 /* static */ Money ScriptEngine::GetRunningCost(EngineID engine_id)
@@ -147,7 +143,7 @@
 	return ::Engine::Get(engine_id)->GetRunningCost();
 }
 
-/* static */ int32 ScriptEngine::GetPower(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetPower(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 	if (GetVehicleType(engine_id) != ScriptVehicle::VT_RAIL && GetVehicleType(engine_id) != ScriptVehicle::VT_ROAD) return -1;
@@ -156,7 +152,7 @@
 	return ::Engine::Get(engine_id)->GetPower();
 }
 
-/* static */ int32 ScriptEngine::GetWeight(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetWeight(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 	if (GetVehicleType(engine_id) != ScriptVehicle::VT_RAIL && GetVehicleType(engine_id) != ScriptVehicle::VT_ROAD) return -1;
@@ -164,20 +160,20 @@
 	return ::Engine::Get(engine_id)->GetDisplayWeight();
 }
 
-/* static */ int32 ScriptEngine::GetMaxTractiveEffort(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetMaxTractiveEffort(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return -1;
 	if (GetVehicleType(engine_id) != ScriptVehicle::VT_RAIL && GetVehicleType(engine_id) != ScriptVehicle::VT_ROAD) return -1;
 	if (IsWagon(engine_id)) return -1;
 
-	return ::Engine::Get(engine_id)->GetDisplayMaxTractiveEffort();
+	return ::Engine::Get(engine_id)->GetDisplayMaxTractiveEffort() / 1000;
 }
 
 /* static */ ScriptDate::Date ScriptEngine::GetDesignDate(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return ScriptDate::DATE_INVALID;
 
-	return (ScriptDate::Date)::Engine::Get(engine_id)->intro_date;
+	return (ScriptDate::Date)::Engine::Get(engine_id)->intro_date.base();
 }
 
 /* static */ ScriptVehicle::VehicleType ScriptEngine::GetVehicleType(EngineID engine_id)
@@ -265,24 +261,19 @@
 	return (ScriptAirport::PlaneType)::AircraftVehInfo(engine_id)->subtype;
 }
 
-/* static */ uint ScriptEngine::GetMaximumOrderDistance(EngineID engine_id)
+/* static */ SQInteger ScriptEngine::GetMaximumOrderDistance(EngineID engine_id)
 {
 	if (!IsValidEngine(engine_id)) return 0;
+	if (GetVehicleType(engine_id) != ScriptVehicle::VT_AIR) return 0;
 
-	switch (GetVehicleType(engine_id)) {
-		case ScriptVehicle::VT_AIR:
-			return ::Engine::Get(engine_id)->GetRange() * ::Engine::Get(engine_id)->GetRange();
-
-		default:
-			return 0;
-	}
+	return (SQInteger)::Engine::Get(engine_id)->GetRange() * ::Engine::Get(engine_id)->GetRange();
 }
 
 /* static */ bool ScriptEngine::EnableForCompany(EngineID engine_id, ScriptCompany::CompanyID company)
 {
 	company = ScriptCompany::ResolveCompanyID(company);
 
-	EnforcePrecondition(false, ScriptObject::GetCompany() == OWNER_DEITY);
+	EnforceDeityMode(false);
 	EnforcePrecondition(false, IsValidEngine(engine_id));
 	EnforcePrecondition(false, company != ScriptCompany::COMPANY_INVALID);
 
@@ -293,7 +284,7 @@
 {
 	company = ScriptCompany::ResolveCompanyID(company);
 
-	EnforcePrecondition(false, ScriptObject::GetCompany() == OWNER_DEITY);
+	EnforceDeityMode(false);
 	EnforcePrecondition(false, IsValidEngine(engine_id));
 	EnforcePrecondition(false, company != ScriptCompany::COMPANY_INVALID);
 
