@@ -22,64 +22,32 @@ class ScriptLog : public ScriptObject {
 	friend class ScriptController;
 
 public:
-	/**
-	 * Log levels; The value is also feed to Debug() lvl.
-	 *  This has no use for you, as script writer.
-	 * @api -all
-	 */
-	enum ScriptLogType {
-		LOG_SQ_ERROR = 0, ///< Squirrel printed an error.
-		LOG_ERROR = 1,    ///< User printed an error.
-		LOG_SQ_INFO = 2,  ///< Squirrel printed some info.
-		LOG_WARNING = 3,  ///< User printed some warning.
-		LOG_INFO = 4,     ///< User printed some info.
-	};
-
-	/**
-	 * Internal representation of the log-data inside the script.
-	 *  This has no use for you, as script writer.
-	 * @api -all
-	 */
-	struct LogData {
-		char **lines;           ///< The log-lines.
-		ScriptLog::ScriptLogType *type; ///< Per line, which type of log it was.
-		int count;              ///< Total amount of log-lines possible.
-		int pos;                ///< Current position in lines.
-		int used;               ///< Total amount of used log-lines.
-	};
-
 	/**
 	 * Print an Info message to the logs.
 	 * @param message The message to log.
 	 * @note Special characters such as U+0000-U+0019 and U+E000-U+E1FF are not supported and removed or replaced by a question mark. This includes newlines and tabs.
 	 */
-	static void Info(const char *message);
+	static void Info(const std::string &message);
 
 	/**
 	 * Print a Warning message to the logs.
 	 * @param message The message to log.
 	 * @note Special characters such as U+0000-U+0019 and U+E000-U+E1FF are not supported and removed or replaced by a question mark. This includes newlines and tabs.
 	 */
-	static void Warning(const char *message);
+	static void Warning(const std::string &message);
 
 	/**
 	 * Print an Error message to the logs.
 	 * @param message The message to log.
 	 * @note Special characters such as U+0000-U+0019 and U+E000-U+E1FF are not supported and removed or replaced by a question mark. This includes newlines and tabs.
 	 */
-	static void Error(const char *message);
-
-	/**
-	 * Free the log pointer.
-	 * @api -all
-	 */
-	static void FreeLogPointer();
+	static void Error(const std::string &message);
 
 private:
 	/**
 	 * Internal command to log the message in a common way.
 	 */
-	static void Log(ScriptLog::ScriptLogType level, const char *message);
+	static void Log(ScriptLogTypes::ScriptLogType level, const std::string &message);
 };
 
 #endif /* SCRIPT_LOG_HPP */
