@@ -18,57 +18,57 @@
 	return ::IsValidTile(t);
 }
 
-/* static */ TileIndex ScriptMap::GetMapSize()
+/* static */ SQInteger ScriptMap::GetMapSize()
 {
-	return ::MapSize();
+	return ::Map::Size();
 }
 
-/* static */ uint32 ScriptMap::GetMapSizeX()
+/* static */ SQInteger ScriptMap::GetMapSizeX()
 {
-	return ::MapSizeX();
+	return ::Map::SizeX();
 }
 
-/* static */ uint32 ScriptMap::GetMapSizeY()
+/* static */ SQInteger ScriptMap::GetMapSizeY()
 {
-	return ::MapSizeY();
+	return ::Map::SizeY();
 }
 
-/* static */ int32 ScriptMap::GetTileX(TileIndex t)
+/* static */ SQInteger ScriptMap::GetTileX(TileIndex t)
 {
 	if (!::IsValidTile(t)) return -1;
 	return ::TileX(t);
 }
 
-/* static */ int32 ScriptMap::GetTileY(TileIndex t)
+/* static */ SQInteger ScriptMap::GetTileY(TileIndex t)
 {
 	if (!::IsValidTile(t)) return -1;
 	return ::TileY(t);
 }
 
-/* static */ TileIndex ScriptMap::GetTileIndex(uint32 x, uint32 y)
+/* static */ TileIndex ScriptMap::GetTileIndex(SQInteger x, SQInteger y)
 {
 	return ::TileXY(x, y);
 }
 
-/* static */ int32 ScriptMap::DistanceManhattan(TileIndex t1, TileIndex t2)
+/* static */ SQInteger ScriptMap::DistanceManhattan(TileIndex t1, TileIndex t2)
 {
 	if (!::IsValidTile(t1) || !::IsValidTile(t2)) return -1;
 	return ::DistanceManhattan(t1, t2);
 }
 
-/* static */ int32 ScriptMap::DistanceMax(TileIndex t1, TileIndex t2)
+/* static */ SQInteger ScriptMap::DistanceMax(TileIndex t1, TileIndex t2)
 {
 	if (!::IsValidTile(t1) || !::IsValidTile(t2)) return -1;
 	return ::DistanceMax(t1, t2);
 }
 
-/* static */ int32 ScriptMap::DistanceSquare(TileIndex t1, TileIndex t2)
+/* static */ SQInteger ScriptMap::DistanceSquare(TileIndex t1, TileIndex t2)
 {
 	if (!::IsValidTile(t1) || !::IsValidTile(t2)) return -1;
 	return ::DistanceSquare(t1, t2);
 }
 
-/* static */ int32 ScriptMap::DistanceFromEdge(TileIndex t)
+/* static */ SQInteger ScriptMap::DistanceFromEdge(TileIndex t)
 {
 	if (!::IsValidTile(t)) return -1;
 	return ::DistanceFromEdge(t);
