@@ -21,8 +21,9 @@
 
 /* static */ bool ScriptStation::IsValidStation(StationID station_id)
 {
+	EnforceDeityOrCompanyModeValid(false);
 	const Station *st = ::Station::GetIfValid(station_id);
-	return st != nullptr && (st->owner == ScriptObject::GetCompany() || ScriptObject::GetCompany() == OWNER_DEITY || st->owner == OWNER_NONE);
+	return st != nullptr && (st->owner == ScriptObject::GetCompany() || ScriptCompanyMode::IsDeity() || st->owner == OWNER_NONE);
 }
 
 /* static */ ScriptCompany::CompanyID ScriptStation::GetOwner(StationID station_id)
@@ -50,7 +51,7 @@ template<bool Tfrom, bool Tvia>
 }
 
 template<bool Tfrom, bool Tvia>
-/* static */ int32 ScriptStation::CountCargoWaiting(StationID station_id,
+/* static */ SQInteger ScriptStation::CountCargoWaiting(StationID station_id,
 		StationID from_station_id, StationID via_station_id, CargoID cargo_id)
 {
 	if (!ScriptStation::IsCargoRequestValid<Tfrom, Tvia>(station_id, from_station_id,
@@ -61,44 +62,44 @@ template<bool Tfrom, bool Tvia>
 	const StationCargoList &cargo_list = ::Station::Get(station_id)->goods[cargo_id].cargo;
 	if (!Tfrom && !Tvia) return cargo_list.TotalCount();
 
-	uint16 cargo_count = 0;
+	uint16_t cargo_count = 0;
 	std::pair<StationCargoList::ConstIterator, StationCargoList::ConstIterator> range = Tvia ?
 				cargo_list.Packets()->equal_range(via_station_id) :
 				std::make_pair(StationCargoList::ConstIterator(cargo_list.Packets()->begin()),
 						StationCargoList::ConstIterator(cargo_list.Packets()->end()));
 	for (StationCargoList::ConstIterator it = range.first; it != range.second; it++) {
 		const CargoPacket *cp = *it;
-		if (!Tfrom || cp->SourceStation() == from_station_id) cargo_count += cp->Count();
+		if (!Tfrom || cp->GetFirstStation() == from_station_id) cargo_count += cp->Count();
 	}
 
 	return cargo_count;
 }
 
-/* static */ int32 ScriptStation::GetCargoWaiting(StationID station_id, CargoID cargo_id)
+/* static */ SQInteger ScriptStation::GetCargoWaiting(StationID station_id, CargoID cargo_id)
 {
 	return CountCargoWaiting<false, false>(station_id, STATION_INVALID, STATION_INVALID, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoWaitingFrom(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoWaitingFrom(StationID station_id,
 		StationID from_station_id, CargoID cargo_id)
 {
 	return CountCargoWaiting<true, false>(station_id, from_station_id, STATION_INVALID, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoWaitingVia(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoWaitingVia(StationID station_id,
 		StationID via_station_id, CargoID cargo_id)
 {
 	return CountCargoWaiting<false, true>(station_id, STATION_INVALID, via_station_id, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoWaitingFromVia(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoWaitingFromVia(StationID station_id,
 		StationID from_station_id, StationID via_station_id, CargoID cargo_id)
 {
 	return CountCargoWaiting<true, true>(station_id, from_station_id, via_station_id, cargo_id);
 }
 
 template<bool Tfrom, bool Tvia>
-/* static */ int32 ScriptStation::CountCargoPlanned(StationID station_id,
+/* static */ SQInteger ScriptStation::CountCargoPlanned(StationID station_id,
 		StationID from_station_id, StationID via_station_id, CargoID cargo_id)
 {
 	if (!ScriptStation::IsCargoRequestValid<Tfrom, Tvia>(station_id, from_station_id,
@@ -115,24 +116,24 @@ template<bool Tfrom, bool Tvia>
 	}
 }
 
-/* static */ int32 ScriptStation::GetCargoPlanned(StationID station_id, CargoID cargo_id)
+/* static */ SQInteger ScriptStation::GetCargoPlanned(StationID station_id, CargoID cargo_id)
 {
 	return CountCargoPlanned<false, false>(station_id, STATION_INVALID, STATION_INVALID, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoPlannedFrom(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoPlannedFrom(StationID station_id,
 		StationID from_station_id, CargoID cargo_id)
 {
 	return CountCargoPlanned<true, false>(station_id, from_station_id, STATION_INVALID, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoPlannedVia(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoPlannedVia(StationID station_id,
 		StationID via_station_id, CargoID cargo_id)
 {
 	return CountCargoPlanned<false, true>(station_id, STATION_INVALID, via_station_id, cargo_id);
 }
 
-/* static */ int32 ScriptStation::GetCargoPlannedFromVia(StationID station_id,
+/* static */ SQInteger ScriptStation::GetCargoPlannedFromVia(StationID station_id,
 		StationID from_station_id, StationID via_station_id, CargoID cargo_id)
 {
 	return CountCargoPlanned<true, true>(station_id, from_station_id, via_station_id, cargo_id);
@@ -146,14 +147,14 @@ template<bool Tfrom, bool Tvia>
 	return ::Station::Get(station_id)->goods[cargo_id].HasRating();
 }
 
-/* static */ int32 ScriptStation::GetCargoRating(StationID station_id, CargoID cargo_id)
+/* static */ SQInteger ScriptStation::GetCargoRating(StationID station_id, CargoID cargo_id)
 {
 	if (!ScriptStation::HasCargoRating(station_id, cargo_id)) return -1;
 
 	return ::ToPercent8(::Station::Get(station_id)->goods[cargo_id].rating);
 }
 
-/* static */ int32 ScriptStation::GetCoverageRadius(ScriptStation::StationType station_type)
+/* static */ SQInteger ScriptStation::GetCoverageRadius(ScriptStation::StationType station_type)
 {
 	if (station_type == STATION_AIRPORT) return -1;
 	if (!HasExactlyOneBit(station_type)) return -1;
@@ -169,21 +170,21 @@ template<bool Tfrom, bool Tvia>
 	}
 }
 
-/* static */ int32 ScriptStation::GetStationCoverageRadius(StationID station_id)
+/* static */ SQInteger ScriptStation::GetStationCoverageRadius(StationID station_id)
 {
 	if (!IsValidStation(station_id)) return -1;
 
 	return Station::Get(station_id)->GetCatchmentRadius();
 }
 
-/* static */ int32 ScriptStation::GetDistanceManhattanToTile(StationID station_id, TileIndex tile)
+/* static */ SQInteger ScriptStation::GetDistanceManhattanToTile(StationID station_id, TileIndex tile)
 {
 	if (!IsValidStation(station_id)) return -1;
 
 	return ScriptMap::DistanceManhattan(tile, GetLocation(station_id));
 }
 
-/* static */ int32 ScriptStation::GetDistanceSquareToTile(StationID station_id, TileIndex tile)
+/* static */ SQInteger ScriptStation::GetDistanceSquareToTile(StationID station_id, TileIndex tile)
 {
 	if (!IsValidStation(station_id)) return -1;
 
@@ -202,7 +203,7 @@ template<bool Tfrom, bool Tvia>
 	if (!IsValidStation(station_id)) return false;
 	if (!HasExactlyOneBit(station_type)) return false;
 
-	return (::Station::Get(station_id)->facilities & station_type) != 0;
+	return (::Station::Get(station_id)->facilities & static_cast<StationFacility>(station_type)) != 0;
 }
 
 /* static */ bool ScriptStation::HasRoadType(StationID station_id, ScriptRoad::RoadType road_type)
@@ -237,6 +238,7 @@ template<bool Tfrom, bool Tvia>
 
 /* static */ bool ScriptStation::OpenCloseAirport(StationID station_id)
 {
+	EnforceCompanyModeValid(false);
 	EnforcePrecondition(false, IsValidStation(station_id));
 	EnforcePrecondition(false, HasStationType(station_id, STATION_AIRPORT));
 
