@@ -10,11 +10,10 @@
 #ifndef SCRIPT_SCANNER_HPP
 #define SCRIPT_SCANNER_HPP
 
-#include <map>
 #include "../fileio_func.h"
-#include "../core/string_compare_type.hpp"
+#include "../string_func.h"
 
-typedef std::map<const char *, class ScriptInfo *, StringCompare> ScriptInfoList; ///< Type for the list of scripts.
+typedef std::map<std::string, class ScriptInfo *, CaseInsensitiveComparator> ScriptInfoList; ///< Type for the list of scripts.
 
 /** Scanner to help finding scripts. */
 class ScriptScanner : public FileScanner {
@@ -56,8 +55,10 @@ public:
 
 	/**
 	 * Get the list of registered scripts to print on the console.
+	 * @param output_iterator The iterator to write the output to.
+	 * @param newest_only Whether to only show the newest scripts.
 	 */
-	std::string GetConsoleList(bool newest_only) const;
+	void GetConsoleList(std::back_insert_iterator<std::string> &output_iterator, bool newest_only) const;
 
 	/**
 	 * Check whether we have a script with the exact characteristics as ci.
@@ -99,7 +100,7 @@ protected:
 	/**
 	 * Get the script name how to store the script in memory.
 	 */
-	virtual void GetScriptName(ScriptInfo *info, char *name, const char *last) = 0;
+	virtual std::string GetScriptName(ScriptInfo *info) = 0;
 
 	/**
 	 * Get the filename to scan for this type of script.
