@@ -60,8 +60,8 @@ Slope GetTileSlope(TileIndex tile, int *h)
 {
 	uint x1 = TileX(tile);
 	uint y1 = TileY(tile);
-	uint x2 = std::min(x1 + 1, MapMaxX());
-	uint y2 = std::min(y1 + 1, MapMaxY());
+	uint x2 = std::min(x1 + 1, Map::MaxX());
+	uint y2 = std::min(y1 + 1, Map::MaxY());
 
 	int hnorth = TileHeight(tile);           // Height of the North corner.
 	int hwest  = TileHeight(TileXY(x2, y1)); // Height of the West corner.
@@ -101,8 +101,8 @@ bool IsTileFlat(TileIndex tile, int *h)
 {
 	uint x1 = TileX(tile);
 	uint y1 = TileY(tile);
-	uint x2 = std::min(x1 + 1, MapMaxX());
-	uint y2 = std::min(y1 + 1, MapMaxY());
+	uint x2 = std::min(x1 + 1, Map::MaxX());
+	uint y2 = std::min(y1 + 1, Map::MaxY());
 
 	uint z = TileHeight(tile);
 	if (TileHeight(TileXY(x2, y1)) != z) return false;
@@ -122,8 +122,8 @@ int GetTileZ(TileIndex tile)
 {
 	uint x1 = TileX(tile);
 	uint y1 = TileY(tile);
-	uint x2 = std::min(x1 + 1, MapMaxX());
-	uint y2 = std::min(y1 + 1, MapMaxY());
+	uint x2 = std::min(x1 + 1, Map::MaxX());
+	uint y2 = std::min(y1 + 1, Map::MaxY());
 
 	return std::min({
 		TileHeight(tile),           // N corner
@@ -142,8 +142,8 @@ int GetTileMaxZ(TileIndex t)
 {
 	uint x1 = TileX(t);
 	uint y1 = TileY(t);
-	uint x2 = std::min(x1 + 1, MapMaxX());
-	uint y2 = std::min(y1 + 1, MapMaxY());
+	uint x2 = std::min(x1 + 1, Map::MaxX());
+	uint y2 = std::min(y1 + 1, Map::MaxY());
 
 	return std::max({
 		TileHeight(t),              // N corner
