@@ -78,7 +78,7 @@ public:
 	 * Get the total covered area.
 	 * @return The area covered by the matrix.
 	 */
-	const TileArea& GetArea() const
+	const TileArea &GetArea() const
 	{
 		return this->area;
 	}
@@ -101,8 +101,8 @@ public:
 		tile_x -= std::min(extend * N, tile_x);
 		tile_y -= std::min(extend * N, tile_y);
 
-		w += std::min(extend * N, MapSizeX() - tile_x - w);
-		h += std::min(extend * N, MapSizeY() - tile_y - h);
+		w += std::min(extend * N, Map::SizeX() - tile_x - w);
+		h += std::min(extend * N, Map::SizeY() - tile_y - h);
 
 		return TileArea(TileXY(tile_x, tile_y), w, h);
 	}
