@@ -27,6 +27,7 @@
 
 #include "../../openttd.h"
 #include "../../debug.h"
+#include "../../error_func.h"
 #include "../../core/geometry_func.hpp"
 #include "../../core/math_func.hpp"
 #include "cocoa_v.h"
@@ -43,7 +44,6 @@
 
 #import <sys/param.h> /* for MAXPATHLEN */
 #import <sys/time.h> /* gettimeofday */
-#include <array>
 
 /* The 10.12 SDK added new names for some enum constants and
  * deprecated the old ones. As there's no functional change in any
@@ -64,13 +64,6 @@
  * Read http://developer.apple.com/releasenotes/Cocoa/Objective-C++.html for more information.
  */
 
-/* On some old versions of MAC OS this may not be defined.
- * Those versions generally only produce code for PPC. So it should be safe to
- * set this to 0. */
-#ifndef kCGBitmapByteOrder32Host
-#define kCGBitmapByteOrder32Host 0
-#endif
-
 bool _cocoa_video_started = false;
 static Palette _local_palette; ///< Current palette to use for drawing.
 
@@ -94,7 +87,8 @@ static const Dimension _default_resolutions[] = {
 };
 
 
-VideoDriver_Cocoa::VideoDriver_Cocoa()
+VideoDriver_Cocoa::VideoDriver_Cocoa(bool uses_hardware_acceleration)
+	: VideoDriver(uses_hardware_acceleration)
 {
 	this->setup         = false;
 	this->buffer_locked = false;
@@ -343,7 +337,7 @@ void VideoDriver_Cocoa::GameSizeChanged()
 	::GameSizeChanged();
 
 	/* We need to store the window size as non-Retina size in
-	* the config file to get same windows size on next start. */
+	 * the config file to get same windows size on next start. */
 	_cur_resolution.width = [ this->cocoaview frame ].size.width;
 	_cur_resolution.height = [ this->cocoaview frame ].size.height;
 }
@@ -407,7 +401,7 @@ bool VideoDriver_Cocoa::MakeWindow(int width, int height)
 		behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
 		[ this->window setCollectionBehavior:behavior ];
 
-		NSButton* fullscreenButton = [ this->window standardWindowButton:NSWindowZoomButton ];
+		NSButton *fullscreenButton = [ this->window standardWindowButton:NSWindowZoomButton ];
 		[ fullscreenButton setAction:@selector(toggleFullScreen:) ];
 		[ fullscreenButton setTarget:this->window ];
 	}
@@ -447,7 +441,7 @@ bool VideoDriver_Cocoa::MakeWindow(int width, int height)
 	CGColorSpaceRelease(this->color_space);
 	this->color_space = CGColorSpaceCreateWithName(kCGColorSpaceSRGB);
 	if (this->color_space == nullptr) this->color_space = CGColorSpaceCreateDeviceRGB();
-	if (this->color_space == nullptr) error("Could not get a valid colour space for drawing.");
+	if (this->color_space == nullptr) FatalError("Could not get a valid colour space for drawing.");
 
 	this->setup = false;
 
@@ -484,11 +478,7 @@ void VideoDriver_Cocoa::InputLoop()
 	_ctrl_pressed = (cur_mods & ( _settings_client.gui.right_mouse_btn_emulation != RMBE_CONTROL ? NSEventModifierFlagControl : NSEventModifierFlagCommand)) != 0;
 	_shift_pressed = (cur_mods & NSEventModifierFlagShift) != 0;
 
-#if defined(_DEBUG)
-	this->fast_forward_key_pressed = _shift_pressed;
-#else
 	this->fast_forward_key_pressed = _tab_is_down;
-#endif
 
 	if (old_ctrl_pressed != _ctrl_pressed) HandleCtrlChanged();
 }
@@ -577,11 +567,11 @@ void VideoDriver_Cocoa::MainLoopReal()
 static FVideoDriver_CocoaQuartz iFVideoDriver_CocoaQuartz;
 
 /** Clear buffer to opaque black. */
-static void ClearWindowBuffer(uint32 *buffer, uint32 pitch, uint32 height)
+static void ClearWindowBuffer(uint32_t *buffer, uint32_t pitch, uint32_t height)
 {
-	uint32 fill = Colour(0, 0, 0).data;
-	for (uint32 y = 0; y < height; y++) {
-		for (uint32 x = 0; x < pitch; x++) {
+	uint32_t fill = Colour(0, 0, 0).data;
+	for (uint32_t y = 0; y < height; y++) {
+		for (uint32_t x = 0; x < pitch; x++) {
 			buffer[y * pitch + x] = fill;
 		}
 	}
@@ -645,7 +635,7 @@ NSView *VideoDriver_CocoaQuartz::AllocateDrawView()
 }
 
 /** Resize the window. */
-void VideoDriver_CocoaQuartz::AllocateBackingStore(bool force)
+void VideoDriver_CocoaQuartz::AllocateBackingStore(bool)
 {
 	if (this->window == nil || this->cocoaview == nil || this->setup) return;
 
@@ -655,14 +645,14 @@ void VideoDriver_CocoaQuartz::AllocateBackingStore(bool force)
 
 	this->window_width = (int)newframe.size.width;
 	this->window_height = (int)newframe.size.height;
-	this->window_pitch = Align(this->window_width, 16 / sizeof(uint32)); // Quartz likes lines that are multiple of 16-byte.
+	this->window_pitch = Align(this->window_width, 16 / sizeof(uint32_t)); // Quartz likes lines that are multiple of 16-byte.
 	this->buffer_depth = BlitterFactory::GetCurrentBlitter()->GetScreenDepth();
 
 	/* Create Core Graphics Context */
 	free(this->window_buffer);
-	this->window_buffer = malloc(this->window_pitch * this->window_height * sizeof(uint32));
+	this->window_buffer = malloc(this->window_pitch * this->window_height * sizeof(uint32_t));
 	/* Initialize with opaque black. */
-	ClearWindowBuffer((uint32 *)this->window_buffer, this->window_pitch, this->window_height);
+	ClearWindowBuffer((uint32_t *)this->window_buffer, this->window_pitch, this->window_height);
 
 	CGContextRelease(this->cgcontext);
 	this->cgcontext = CGBitmapContextCreate(
@@ -675,7 +665,7 @@ void VideoDriver_CocoaQuartz::AllocateBackingStore(bool force)
 		kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Host
 	);
 
-	assert(this->cgcontext != NULL);
+	assert(this->cgcontext != nullptr);
 	CGContextSetShouldAntialias(this->cgcontext, FALSE);
 	CGContextSetAllowsAntialiasing(this->cgcontext, FALSE);
 	CGContextSetInterpolationQuality(this->cgcontext, kCGInterpolationNone);
@@ -683,7 +673,7 @@ void VideoDriver_CocoaQuartz::AllocateBackingStore(bool force)
 	if (this->buffer_depth == 8) {
 		free(this->pixel_buffer);
 		this->pixel_buffer = malloc(this->window_width * this->window_height);
-		if (this->pixel_buffer == nullptr) usererror("Out of memory allocating pixel buffer");
+		if (this->pixel_buffer == nullptr) UserError("Out of memory allocating pixel buffer");
 	} else {
 		free(this->pixel_buffer);
 		this->pixel_buffer = nullptr;
@@ -710,9 +700,9 @@ void VideoDriver_CocoaQuartz::AllocateBackingStore(bool force)
  */
 void VideoDriver_CocoaQuartz::BlitIndexedToView32(int left, int top, int right, int bottom)
 {
-	const uint32 *pal   = this->palette;
-	const uint8  *src   = (uint8*)this->pixel_buffer;
-	uint32       *dst   = (uint32*)this->window_buffer;
+	const uint32_t *pal   = this->palette;
+	const uint8_t  *src   = (uint8_t*)this->pixel_buffer;
+	uint32_t       *dst   = (uint32_t*)this->window_buffer;
 	uint          width = this->window_width;
 	uint          pitch = this->window_pitch;
 
@@ -729,10 +719,10 @@ void VideoDriver_CocoaQuartz::UpdatePalette(uint first_color, uint num_colors)
 	if (this->buffer_depth != 8) return;
 
 	for (uint i = first_color; i < first_color + num_colors; i++) {
-		uint32 clr = 0xff000000;
-		clr |= (uint32)_local_palette.palette[i].r << 16;
-		clr |= (uint32)_local_palette.palette[i].g << 8;
-		clr |= (uint32)_local_palette.palette[i].b;
+		uint32_t clr = 0xff000000;
+		clr |= (uint32_t)_local_palette.palette[i].r << 16;
+		clr |= (uint32_t)_local_palette.palette[i].g << 8;
+		clr |= (uint32_t)_local_palette.palette[i].b;
 		this->palette[i] = clr;
 	}
 
