@@ -17,12 +17,13 @@
 #include "../gfx_func.h"
 #include "../gfxinit.h"
 #include "../progress.h"
+#include "../rev.h"
 #include "../thread.h"
 #include "../window_func.h"
 #include "video_driver.hpp"
 
-bool _video_hw_accel; ///< Whether to consider hardware accelerated video drivers.
-bool _video_vsync; ///< Whether we should use vsync (only if _video_hw_accel is enabled).
+bool _video_hw_accel; ///< Whether to consider hardware accelerated video drivers on startup.
+bool _video_vsync; ///< Whether we should use vsync (only if active video driver supports HW acceleration).
 
 void VideoDriver::GameLoop()
 {
@@ -180,3 +181,12 @@ void VideoDriver::SleepTillNextTick()
 		std::this_thread::sleep_for(next_tick - now);
 	}
 }
+
+/**
+ * Get the caption to use for the game's title bar.
+ * @return The caption.
+ */
+/* static */ std::string VideoDriver::GetCaption()
+{
+	return fmt::format("OpenTTD {}", _openttd_revision);
+}
