@@ -16,13 +16,12 @@
 #include "../gfx_func.h"
 #include "../settings_type.h"
 #include "../zoom_type.h"
+#include "../network/network_func.h"
 #include <atomic>
 #include <chrono>
 #include <condition_variable>
 #include <mutex>
 #include <thread>
-#include <vector>
-#include <functional>
 
 extern std::string _ini_videodriver;
 extern std::vector<Dimension> _resolutions;
@@ -37,7 +36,7 @@ class VideoDriver : public Driver {
 	const uint DEFAULT_WINDOW_HEIGHT = 480u; ///< Default window height.
 
 public:
-	VideoDriver() : fast_forward_key_pressed(false), fast_forward_via_key(false), is_game_threaded(true) {}
+	VideoDriver(bool uses_hardware_acceleration = false) : fast_forward_key_pressed(false), fast_forward_via_key(false), is_game_threaded(true), uses_hardware_acceleration(uses_hardware_acceleration) {}
 
 	/**
 	 * Mark a particular area dirty.
@@ -72,7 +71,7 @@ public:
 	 * Change the vsync setting.
 	 * @param vsync The new setting.
 	 */
-	virtual void ToggleVsync(bool vsync) {}
+	virtual void ToggleVsync([[maybe_unused]] bool vsync) {}
 
 	/**
 	 * Callback invoked after the blitter was changed.
@@ -142,7 +141,7 @@ public:
 	 * Get a pointer to the animation buffer of the video back-end.
 	 * @return Pointer to the buffer or nullptr if no animation buffer is supported.
 	 */
-	virtual uint8 *GetAnimBuffer()
+	virtual uint8_t *GetAnimBuffer()
 	{
 		return nullptr;
 	}
@@ -199,10 +198,13 @@ public:
 	/**
 	 * Get the currently active instance of the video driver.
 	 */
-	static VideoDriver *GetInstance() {
+	static VideoDriver *GetInstance()
+	{
 		return static_cast<VideoDriver*>(*DriverFactoryBase::GetActiveDriver(Driver::DT_VIDEO));
 	}
 
+	static std::string GetCaption();
+
 	/**
 	 * Helper struct to ensure the video buffer is locked and ready for drawing. The destructor
 	 * will make sure the buffer is unlocked no matter how the scope is exited.
@@ -260,7 +262,8 @@ protected:
 	 * Make sure the video buffer is ready for drawing.
 	 * @returns True if the video buffer has to be unlocked.
 	 */
-	virtual bool LockVideoBuffer() {
+	virtual bool LockVideoBuffer()
+	{
 		return false;
 	}
 
@@ -309,6 +312,12 @@ protected:
 
 	std::chrono::steady_clock::duration GetGameInterval()
 	{
+#ifdef DEBUG_DUMP_COMMANDS
+		/* When replaying, run as fast as we can. */
+		extern bool _ddc_fastforward;
+		if (_ddc_fastforward) return std::chrono::microseconds(0);
+#endif /* DEBUG_DUMP_COMMANDS */
+
 		/* If we are paused, run on normal speed. */
 		if (_pause_mode) return std::chrono::milliseconds(MILLISECONDS_PER_TICK);
 		/* Infinite speed, as quickly as you can. */
@@ -320,7 +329,7 @@ protected:
 	std::chrono::steady_clock::duration GetDrawInterval()
 	{
 		/* If vsync, draw interval is decided by the display driver */
-		if (_video_vsync && _video_hw_accel) return std::chrono::microseconds(0);
+		if (_video_vsync && this->uses_hardware_acceleration) return std::chrono::microseconds(0);
 		return std::chrono::microseconds(1000000 / _settings_client.gui.refresh_rate);
 	}
 
@@ -353,6 +362,8 @@ protected:
 	std::mutex game_state_mutex;
 	std::mutex game_thread_wait_mutex;
 
+	bool uses_hardware_acceleration;
+
 	static void GameThreadThunk(VideoDriver *drv);
 
 private:
