@@ -39,6 +39,7 @@ void MakeWaterKeepingClass(TileIndex tile, Owner o);
 void CheckForDockingTile(TileIndex t);
 
 bool RiverModifyDesertZone(TileIndex tile, void *data);
+void MakeRiverAndModifyDesertZoneAround(TileIndex tile);
 static const uint RIVER_OFFSET_DESERT_DISTANCE = 5; ///< Circular tile search radius to create non-desert around a river tile.
 
 bool IsWateredTile(TileIndex tile, Direction from);
@@ -48,7 +49,7 @@ bool IsWateredTile(TileIndex tile, Direction from);
  * @param num Number of canal tiles.
  * @return Total cost.
  */
-static inline Money CanalMaintenanceCost(uint32 num)
+inline Money CanalMaintenanceCost(uint32_t num)
 {
 	return (_price[PR_INFRASTRUCTURE_WATER] * num * (1 + IntSqrt(num))) >> 6; // 6 bits scaling.
 }
