@@ -11,7 +11,7 @@
 #define WIDGETS_COMPANY_WIDGET_H
 
 /** Widgets of the #CompanyWindow class. */
-enum CompanyWidgets {
+enum CompanyWidgets : WidgetID {
 	WID_C_CAPTION,                    ///< Caption of the window.
 
 	WID_C_FACE,                       ///< View of the face.
@@ -26,16 +26,11 @@ enum CompanyWidgets {
 	WID_C_DESC_INFRASTRUCTURE,        ///< Infrastructure.
 	WID_C_DESC_INFRASTRUCTURE_COUNTS, ///< Infrastructure count.
 
-	WID_C_SELECT_DESC_OWNERS,         ///< Owners.
-	WID_C_DESC_OWNERS,                ///< Owner in Owners.
-
 	WID_C_SELECT_BUTTONS,             ///< Selection widget for the button bar.
 	WID_C_NEW_FACE,                   ///< Button to make new face.
 	WID_C_COLOUR_SCHEME,              ///< Button to change colour scheme.
 	WID_C_PRESIDENT_NAME,             ///< Button to change president name.
 	WID_C_COMPANY_NAME,               ///< Button to change company name.
-	WID_C_BUY_SHARE,                  ///< Button to buy a share.
-	WID_C_SELL_SHARE,                 ///< Button to sell a share.
 
 	WID_C_SELECT_VIEW_BUILD_HQ,       ///< Panel about HQ.
 	WID_C_VIEW_HQ,                    ///< Button to view the HQ.
@@ -49,6 +44,9 @@ enum CompanyWidgets {
 	WID_C_SELECT_GIVE_MONEY,          ///< Selection widget for the give money button.
 	WID_C_GIVE_MONEY,                 ///< Button to give money.
 
+	WID_C_SELECT_HOSTILE_TAKEOVER,    ///< Selection widget for the hostile takeover button.
+	WID_C_HOSTILE_TAKEOVER,           ///< Button to hostile takeover another company.
+
 	WID_C_HAS_PASSWORD,               ///< Has company password lock.
 	WID_C_SELECT_MULTIPLAYER,         ///< Multiplayer selection panel.
 	WID_C_COMPANY_PASSWORD,           ///< Button to set company password.
@@ -56,7 +54,7 @@ enum CompanyWidgets {
 };
 
 /** Widgets of the #CompanyFinancesWindow class. */
-enum CompanyFinancesWidgets {
+enum CompanyFinancesWidgets : WidgetID {
 	WID_CF_CAPTION,        ///< Caption of the window.
 	WID_CF_TOGGLE_SIZE,    ///< Toggle windows size.
 	WID_CF_SEL_PANEL,      ///< Select panel or nothing.
@@ -80,7 +78,7 @@ enum CompanyFinancesWidgets {
 
 
 /** Widgets of the #SelectCompanyLiveryWindow class. */
-enum SelectCompanyLiveryWidgets {
+enum SelectCompanyLiveryWidgets : WidgetID {
 	WID_SCL_CAPTION,          ///< Caption of window.
 	WID_SCL_CLASS_GENERAL,    ///< Class general.
 	WID_SCL_CLASS_RAIL,       ///< Class rail.
@@ -104,7 +102,7 @@ enum SelectCompanyLiveryWidgets {
  * Do not change the order of the widgets from WID_SCMF_HAS_MOUSTACHE_EARRING to WID_SCMF_GLASSES_R,
  * this order is needed for the WE_CLICK event of DrawFaceStringLabel().
  */
-enum SelectCompanyManagerFaceWidgets {
+enum SelectCompanyManagerFaceWidgets : WidgetID {
 	WID_SCMF_CAPTION,                    ///< Caption of window.
 	WID_SCMF_TOGGLE_LARGE_SMALL,         ///< Toggle for large or small.
 	WID_SCMF_SELECT_FACE,                ///< Select face.
@@ -172,7 +170,7 @@ enum SelectCompanyManagerFaceWidgets {
 };
 
 /** Widgets of the #CompanyInfrastructureWindow class. */
-enum CompanyInfrastructureWidgets {
+enum CompanyInfrastructureWidgets : WidgetID {
 	WID_CI_CAPTION,       ///< Caption of window.
 	WID_CI_RAIL_DESC,     ///< Description of rail.
 	WID_CI_RAIL_COUNT,    ///< Count of rail.
@@ -189,7 +187,7 @@ enum CompanyInfrastructureWidgets {
 };
 
 /** Widgets of the #BuyCompanyWindow class. */
-enum BuyCompanyWidgets {
+enum BuyCompanyWidgets : WidgetID {
 	WID_BC_CAPTION,  ///< Caption of window.
 	WID_BC_FACE,     ///< Face button.
 	WID_BC_QUESTION, ///< Question text.
