@@ -11,7 +11,7 @@
 #define WIDGETS_CONSOLE_WIDGET_H
 
 /** Widgets of the #IConsoleWindow class. */
-enum ConsoleWidgets {
+enum ConsoleWidgets : WidgetID {
 	WID_C_BACKGROUND, ///< Background of the console.
 };
 
