@@ -11,7 +11,7 @@
 #define WIDGETS_DATE_WIDGET_H
 
 /** Widgets of the #SetDateWindow class. */
-enum SetDateWidgets {
+enum SetDateWidgets : WidgetID {
 	WID_SD_DAY,      ///< Dropdown for the day.
 	WID_SD_MONTH,    ///< Dropdown for the month.
 	WID_SD_YEAR,     ///< Dropdown for the year.
