@@ -11,14 +11,14 @@
 #define WIDGETS_DOCK_WIDGET_H
 
 /** Widgets of the #BuildDocksDepotWindow class. */
-enum BuildDockDepotWidgets {
+enum BuildDockDepotWidgets : WidgetID {
 	WID_BDD_BACKGROUND, ///< Background of the window.
 	WID_BDD_X,          ///< X-direction button.
 	WID_BDD_Y,          ///< Y-direction button.
 };
 
 /** Widgets of the #BuildDocksToolbarWindow class. */
-enum DockToolbarWidgets {
+enum DockToolbarWidgets : WidgetID {
 	WID_DT_CANAL,          ///< Build canal button.
 	WID_DT_LOCK,           ///< Build lock button.
 	WID_DT_DEMOLISH,       ///< Demolish aka dynamite button.
