@@ -11,7 +11,7 @@
 #define WIDGETS_ERROR_WIDGET_H
 
 /** Widgets of the #ErrmsgWindow class. */
-enum ErrorMessageWidgets {
+enum ErrorMessageWidgets : WidgetID {
 	WID_EM_CAPTION, ///< Caption of the window.
 	WID_EM_FACE,    ///< Error title.
 	WID_EM_MESSAGE, ///< Error message.
