@@ -5,7 +5,7 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file ai_widget.h Types related to the ai widgets. */
+/** @file game_widget.h Types related to the GS widgets. */
 
 #ifndef WIDGETS_GS_WIDGET_H
 #define WIDGETS_GS_WIDGET_H
@@ -13,16 +13,18 @@
 #include "../textfile_type.h"
 
 /** Widgets of the #GSConfigWindow class. */
-enum GSConfigWidgets {
+enum GSConfigWidgets : WidgetID {
 	WID_GSC_BACKGROUND,       ///< Window background.
 	WID_GSC_GSLIST,           ///< List with current selected Game Script.
 	WID_GSC_SETTINGS,         ///< Panel to draw the Game Script settings on
 	WID_GSC_SCROLLBAR,        ///< Scrollbar to scroll through the selected AIs.
 	WID_GSC_CHANGE,           ///< Select another Game Script button.
+	WID_GSC_OPEN_URL,         ///< Open GS URL.
 	WID_GSC_TEXTFILE,         ///< Open GS readme, changelog (+1) or license (+2).
-	WID_GSC_CONTENT_DOWNLOAD = WID_GSC_TEXTFILE + TFT_END, ///< Download content button.
-	WID_GSC_ACCEPT,           ///< Accept ("Close") button
+	WID_GSC_CONTENT_DOWNLOAD = WID_GSC_TEXTFILE + TFT_CONTENT_END, ///< Download content button.
 	WID_GSC_RESET,            ///< Reset button.
+
+	WID_GSC_SETTING_DROPDOWN = -1, ///< Dynamically created dropdown for changing setting value.
 };
 
 #endif /* WIDGETS_GS_WIDGET_H */
