@@ -14,37 +14,24 @@
 #include "../company_type.h"
 
 /** Widgets of the #GraphLegendWindow class. */
-enum GraphLegendWidgets {
+enum GraphLegendWidgets : WidgetID {
 	WID_GL_BACKGROUND,    ///< Background of the window.
 
 	WID_GL_FIRST_COMPANY, ///< First company in the legend.
 	WID_GL_LAST_COMPANY = WID_GL_FIRST_COMPANY + MAX_COMPANIES - 1, ///< Last company in the legend.
 };
 
-/** Widgets of the #OperatingProfitGraphWindow class, #IncomeGraphWindow class, #DeliveredCargoGraphWindow class, and #CompanyValueGraphWindow class. */
-enum CompanyValueWidgets {
-	WID_CV_KEY_BUTTON, ///< Key button.
-	WID_CV_BACKGROUND, ///< Background of the window.
-	WID_CV_GRAPH,      ///< Graph itself.
-	WID_CV_RESIZE,     ///< Resize button.
-};
+/** Widgets of the #BaseGraphWindow class and derived classes. */
+enum GraphWidgets : WidgetID {
+	WID_GRAPH_KEY_BUTTON, ///< Key button.
+	WID_GRAPH_BACKGROUND, ///< Background of the window.
+	WID_GRAPH_GRAPH,      ///< Graph itself.
+	WID_GRAPH_RESIZE,     ///< Resize button.
+	WID_GRAPH_HEADER,     ///< Header.
+	WID_GRAPH_FOOTER,     ///< Footer.
 
-/** Widget of the #PerformanceHistoryGraphWindow class. */
-enum PerformanceHistoryGraphWidgets {
-	WID_PHG_KEY,                  ///< Key button.
 	WID_PHG_DETAILED_PERFORMANCE, ///< Detailed performance.
-	WID_PHG_BACKGROUND,           ///< Background of the window.
-	WID_PHG_GRAPH,                ///< Graph itself.
-	WID_PHG_RESIZE,               ///< Resize button.
-};
 
-/** Widget of the #PaymentRatesGraphWindow class. */
-enum CargoPaymentRatesWidgets {
-	WID_CPR_BACKGROUND,      ///< Background of the window.
-	WID_CPR_HEADER,          ///< Header.
-	WID_CPR_GRAPH,           ///< Graph itself.
-	WID_CPR_RESIZE,          ///< Resize button.
-	WID_CPR_FOOTER,          ///< Footer.
 	WID_CPR_ENABLE_CARGOES,  ///< Enable cargoes button.
 	WID_CPR_DISABLE_CARGOES, ///< Disable cargoes button.
 	WID_CPR_MATRIX,          ///< Cargo list.
@@ -52,7 +39,7 @@ enum CargoPaymentRatesWidgets {
 };
 
 /** Widget of the #PerformanceRatingDetailWindow class. */
-enum PerformanceRatingDetailsWidgets {
+enum PerformanceRatingDetailsWidgets : WidgetID {
 	WID_PRD_SCORE_FIRST, ///< First entry in the score list.
 	WID_PRD_SCORE_LAST = WID_PRD_SCORE_FIRST + (SCORE_END - SCORE_BEGIN) - 1, ///< Last entry in the score list.
 
