@@ -11,8 +11,12 @@
 #define WIDGETS_SETTINGS_WIDGET_H
 
 /** Widgets of the #GameOptionsWindow class. */
-enum GameOptionsWidgets {
-	WID_GO_BACKGROUND,             ///< Background of the window.
+enum GameOptionsWidgets : WidgetID {
+	WID_GO_TAB_GENERAL,            ///< General tab.
+	WID_GO_TAB_GRAPHICS,           ///< Graphics tab.
+	WID_GO_TAB_SOUND,              ///< Sound tab.
+	WID_GO_TAB_SOCIAL,             ///< Social tab.
+	WID_GO_TAB_SELECTION,          ///< Background of the tab selection.
 	WID_GO_CURRENCY_DROPDOWN,      ///< Currency dropdown.
 	WID_GO_DISTANCE_DROPDOWN,      ///< Measuring unit dropdown.
 	WID_GO_AUTOSAVE_DROPDOWN,      ///< Dropdown to say how often to autosave.
@@ -22,28 +26,42 @@ enum GameOptionsWidgets {
 	WID_GO_GUI_SCALE,              ///< GUI Scale slider.
 	WID_GO_GUI_SCALE_AUTO,         ///< Autodetect GUI scale button.
 	WID_GO_GUI_SCALE_BEVEL_BUTTON, ///< Toggle for chunky bevels.
+	WID_GO_GUI_FONT_SPRITE,        ///< Toggle whether to prefer the sprite font over TTF fonts.
+	WID_GO_GUI_FONT_AA,            ///< Toggle whether to anti-alias fonts.
 	WID_GO_BASE_GRF_DROPDOWN,      ///< Use to select a base GRF.
-	WID_GO_BASE_GRF_STATUS,        ///< Info about missing files etc.
+	WID_GO_BASE_GRF_PARAMETERS,    ///< Base GRF parameters.
+	WID_GO_BASE_GRF_OPEN_URL,      ///< Open base GRF URL.
 	WID_GO_BASE_GRF_TEXTFILE,      ///< Open base GRF readme, changelog (+1) or license (+2).
-	WID_GO_BASE_GRF_DESCRIPTION = WID_GO_BASE_GRF_TEXTFILE + TFT_END,     ///< Description of selected base GRF.
+	WID_GO_BASE_GRF_DESCRIPTION = WID_GO_BASE_GRF_TEXTFILE + TFT_CONTENT_END,     ///< Description of selected base GRF.
 	WID_GO_BASE_SFX_DROPDOWN,      ///< Use to select a base SFX.
+	WID_GO_TEXT_SFX_VOLUME,        ///< Sound effects volume label.
 	WID_GO_BASE_SFX_VOLUME,        ///< Change sound effects volume.
+	WID_GO_BASE_SFX_OPEN_URL,      ///< Open base SFX URL.
 	WID_GO_BASE_SFX_TEXTFILE,      ///< Open base SFX readme, changelog (+1) or license (+2).
-	WID_GO_BASE_SFX_DESCRIPTION = WID_GO_BASE_SFX_TEXTFILE + TFT_END,     ///< Description of selected base SFX.
+	WID_GO_BASE_SFX_DESCRIPTION = WID_GO_BASE_SFX_TEXTFILE + TFT_CONTENT_END,     ///< Description of selected base SFX.
 	WID_GO_BASE_MUSIC_DROPDOWN,    ///< Use to select a base music set.
+	WID_GO_TEXT_MUSIC_VOLUME,      ///< Music volume label.
 	WID_GO_BASE_MUSIC_VOLUME,      ///< Change music volume.
 	WID_GO_BASE_MUSIC_JUKEBOX,     ///< Open the jukebox.
-	WID_GO_BASE_MUSIC_STATUS,      ///< Info about corrupted files etc.
+	WID_GO_BASE_MUSIC_OPEN_URL,    ///< Open base music URL.
 	WID_GO_BASE_MUSIC_TEXTFILE,    ///< Open base music readme, changelog (+1) or license (+2).
-	WID_GO_BASE_MUSIC_DESCRIPTION = WID_GO_BASE_MUSIC_TEXTFILE + TFT_END, ///< Description of selected base music set.
+	WID_GO_BASE_MUSIC_DESCRIPTION = WID_GO_BASE_MUSIC_TEXTFILE + TFT_CONTENT_END, ///< Description of selected base music set.
 	WID_GO_VIDEO_ACCEL_BUTTON,     ///< Toggle for video acceleration.
 	WID_GO_VIDEO_VSYNC_BUTTON,     ///< Toggle for video vsync.
 	WID_GO_REFRESH_RATE_DROPDOWN,  ///< Dropdown for all available refresh rates.
 	WID_GO_VIDEO_DRIVER_INFO,      ///< Label showing details about the current video driver.
+	WID_GO_SURVEY_SEL,             ///< Selection to hide survey if no JSON library is compiled in.
+	WID_GO_SURVEY_PARTICIPATE_BUTTON, ///< Toggle for participating in the automated survey.
+	WID_GO_SURVEY_LINK_BUTTON,     ///< Button to open browser to go to the survey website.
+	WID_GO_SURVEY_PREVIEW_BUTTON,  ///< Button to open a preview window with the survey results
+	WID_GO_SOCIAL_PLUGINS,         ///< Main widget handling the social plugins.
+	WID_GO_SOCIAL_PLUGIN_TITLE,    ///< Title of the frame of the social plugin.
+	WID_GO_SOCIAL_PLUGIN_PLATFORM, ///< Platform of the social plugin.
+	WID_GO_SOCIAL_PLUGIN_STATE,    ///< State of the social plugin.
 };
 
 /** Widgets of the #GameSettingsWindow class. */
-enum GameSettingsWidgets {
+enum GameSettingsWidgets : WidgetID {
 	WID_GS_FILTER,             ///< Text filter.
 	WID_GS_OPTIONSPANEL,       ///< Panel widget containing the option lists.
 	WID_GS_SCROLLBAR,          ///< Scrollbar.
@@ -55,10 +73,12 @@ enum GameSettingsWidgets {
 	WID_GS_RESTRICT_TYPE,      ///< Label upfront to the type drop-down box to restrict the list of settings to show
 	WID_GS_RESTRICT_DROPDOWN,  ///< The drop down box to restrict the list of settings
 	WID_GS_TYPE_DROPDOWN,      ///< The drop down box to choose client/game/company/all settings
+
+	WID_GS_SETTING_DROPDOWN = -1, ///< Dynamically created dropdown for changing setting value.
 };
 
 /** Widgets of the #CustomCurrencyWindow class. */
-enum CustomCurrencyWidgets {
+enum CustomCurrencyWidgets : WidgetID {
 	WID_CC_RATE_DOWN,      ///< Down button.
 	WID_CC_RATE_UP,        ///< Up button.
 	WID_CC_RATE,           ///< Rate of currency.
