@@ -11,12 +11,13 @@
 #define WIDGETS_WAYPOINT_WIDGET_H
 
 /** Widgets of the #WaypointWindow class. */
-enum WaypointWidgets {
+enum WaypointWidgets : WidgetID {
 	WID_W_CAPTION,       ///< Caption of window.
 	WID_W_VIEWPORT,      ///< The viewport on this waypoint.
 	WID_W_CENTER_VIEW,   ///< Center the main view on this waypoint.
 	WID_W_RENAME,        ///< Rename this waypoint.
 	WID_W_SHOW_VEHICLES, ///< Show the vehicles visiting this waypoint.
+	WID_W_CATCHMENT,     ///< Coverage button.
 };
 
 #endif /* WIDGETS_WAYPOINT_WIDGET_H */
